"""Tests for Inputparser"""

from MAST.parsers.inputparser import InputParser

import unittest
from unittest import SkipTest
import os
import time
import MAST
import pymatgen
from MAST.utility import dirutil
from MAST.utility import InputOptions
from MAST.utility import MASTFile
from MAST.utility import MASTError
import numpy as np

testname="inputparser_test"
testdir = dirutil.get_test_dir(testname)

class TestInputparser(unittest.TestCase):

    def setUp(self):
        os.chdir(testdir)

    def tearDown(self):
        pass

    def test___init__(self):
        myip = InputParser(inputfile="long_input.inp")
        s_dict = dict({\
                'mast' : myip.parse_mast_section,
                'structure' : myip.parse_structure_section,
                'ingredients' : myip.parse_ingredients_section,
                'defects' : myip.parse_defects_section,
                'recipe' : myip.parse_recipe_section,
                'neb' : myip.parse_neb_section,
                'chemical_potentials' : myip.parse_chemical_potentials_section,
                'summary' : myip.parse_summary_section,
                               })
        self.assertItemsEqual(myip.section_parsers, s_dict)
        #self.testclass.__init__(**kwargs)

    def test_parse(self):
        myip = InputParser(inputfile="long_input.inp")
        myoptions = myip.parse()
        self.assertItemsEqual(myoptions.options.keys(),['mast','structure','defects','recipe','ingredients','neb','chemical_potentials'])
        #ofile = MASTFile()
        #ofile.data = str(myoptions)
        #ofile.to_file(os.path.join(os.getcwd(),"long_input_options_output.txt"))
        #self.testclass.parse()

    def test_parse_mast_section(self):
        myip = InputParser(inputfile="long_input.inp")
        minput = MASTFile("%s/mast_lines.txt" % testdir)
        cleanlines = list()
        for line in minput.data:
            cleanlines.append(line.strip())
        myoptions = InputOptions()
        myip.parse_mast_section('mast',cleanlines,myoptions)
        mdict=dict()
        mdict['mast']=dict()
        mdict['mast']['system_name'] = "SystemName!"
        self.assertItemsEqual(myoptions.options.keys(), ['mast'])
        self.assertItemsEqual(myoptions.options['mast'].keys(), ['system_name','mast_auto_correct'])
        self.assertEqual(myoptions.options['mast']['system_name'],mdict['mast']['system_name']) 
        self.assertEqual(myoptions.get_item('mast', 'mast_auto_correct'), "True")
        #print myoptions.options['mast']
        #self.testclass.parse_mast_section(section_name, section_content, options)

    def test_parse_structure_section_noposfile(self):
        myip = InputParser(inputfile="long_input.inp")
        minput = MASTFile("%s/structure_noposfile_lines.txt" % testdir)
        cleanlines = list()
        for line in minput.data:
            cleanlines.append(line.strip())
        myoptions = InputOptions()
        myip.parse_structure_section('structure',cleanlines,myoptions)
        #print myoptions
        self.assertEqual(myoptions.options['structure']['atom_list'],['Al','Mg','Al','Fe'])
        self.assertEqual(myoptions.options['structure']['coord_type'],'fractional')
        self.assertTrue(np.array_equal(myoptions.options['structure']['lattice'],np.array([[3.5,1,2],[3,3.5,4],[5,6,3.5]])))
        self.assertTrue(np.array_equal(myoptions.options['structure']['coordinates'],np.array([[0,0,0],[0.5,0.5,0],[0,0.5,0.5],[0.5,0,0.5]])))
        self.assertItemsEqual(myoptions.options['structure']['element_map'],dict({'X1':'Al','X2':'Mg','X3':'Fe'}))

        #mdict=dict()
        #mdict['structure'] = dict()
        #mdict['structure']['posfile'] = os.path.join(testdir, "POSCAR_startpos")
        #self.assertItemsEqual(myoptions.options.keys(), ['structure'])
        #self.assertEqual(myoptions.options['structure']['posfile'],mdict['structure']['posfile']) 
        #self.testclass.parse_structure_section(section_name, section_content, options)
    
    def test_parse_structure_section_posfile(self):
        myip = InputParser(inputfile="long_input.inp")
        minput = MASTFile("%s/structure_posfile_lines.txt" % testdir)
        cleanlines = list()
        for line in minput.data:
            cleanlines.append(line.strip())
        myoptions = InputOptions()
        myip.parse_structure_section('structure',cleanlines,myoptions)
        #print myoptions
        mdict=dict()
        mdict['structure'] = dict()
        mdict['structure']['posfile'] = os.path.join(testdir, "POSCAR_startpos")
        self.assertItemsEqual(myoptions.options.keys(), ['structure'])
        self.assertEqual(myoptions.options['structure']['posfile'],mdict['structure']['posfile']) 
        myip = InputParser(inputfile="long_input.inp")
        minput = MASTFile("%s/structure_posfile_lines_nosuch.txt" % testdir)
        cleanlines = list()
        for line in minput.data:
            cleanlines.append(line.strip())
        myoptions = InputOptions()
        self.assertRaises(MASTError, myip.parse_structure_section,'structure',cleanlines,myoptions)
        #self.testclass.parse_structure_section(section_name, section_content, options)

    def test_parse_defects_section(self):
        myip = InputParser(inputfile="long_input.inp")
        minput = MASTFile("%s/defects_lines.txt" % testdir)
        cleanlines = list()
        for line in minput.data:
            cleanlines.append(line.strip())
        myoptions = InputOptions()
        myip.parse_defects_section('defects',cleanlines,myoptions)
        print myoptions.options['defects']
        #Test coordinates first, to avoid dictionary-equal error on comparing arryas
        self.assertTrue(np.array_equal(myoptions.options['defects']['defects']['defect_3']['subdefect_1']['coordinates'],np.array([0.,0.,0.],'float')))
        self.assertTrue(np.array_equal(myoptions.options['defects']['defects']['Vac@Al-Sub@Fe']['subdefect_1']['coordinates'],np.array([0.,0.,0.],'float')))
        self.assertTrue(np.array_equal(myoptions.options['defects']['defects']['Vac@Al-Sub@Fe']['subdefect_2']['coordinates'],np.array([0.5,0.5,0.],'float')))
        self.assertTrue(np.array_equal(myoptions.options['defects']['defects']['AntiGe@Fe']['subdefect_1']['coordinates'],np.array([0.1,0.3,0.4],'float')))
        self.assertTrue(np.array_equal(myoptions.options['defects']['defects']['MgInt']['subdefect_1']['coordinates'],np.array([0.,0.2,0.3],'float')))
        self.assertItemsEqual(myoptions.options['defects']['defects']['Vac@Al-Sub@Fe']['phonon'].keys(),['host1','sub2'])
        #Have to blank out the numpy arrays so that the rest of the diff'ing will work
        myoptions.options['defects']['defects']['defect_3']['subdefect_1']['coordinates']='blank'
        myoptions.options['defects']['defects']['Vac@Al-Sub@Fe']['subdefect_1']['coordinates']='blank'
        myoptions.options['defects']['defects']['Vac@Al-Sub@Fe']['subdefect_2']['coordinates']='blank'
        myoptions.options['defects']['defects']['AntiGe@Fe']['subdefect_1']['coordinates']='blank'
        myoptions.options['defects']['defects']['MgInt']['subdefect_1']['coordinates']='blank'
        mdict=dict()
        mdict['coord_type']='fractional'
        mdict['num_defects']=4
        mdict['defects']=dict()
        mdict['defects']['defect_3']=dict()
        mdict['defects']['defect_3']['threshold']=0.0001
        mdict['defects']['defect_3']['charge']=[0] #charge of 0
        mdict['defects']['defect_3']['coord_type']='fractional'
        mdict['defects']['defect_3']['subdefect_1']=dict()
        mdict['defects']['defect_3']['subdefect_1']['symbol']='Al'
        mdict['defects']['defect_3']['subdefect_1']['type']='vacancy'
        mdict['defects']['defect_3']['subdefect_1']['coordinates']='blank' #np.array([0.,0.,0.],'float')
        mdict['defects']['defect_3']['phonon']={}
        mdict['defects']['Vac@Al-Sub@Fe']=dict()
        mdict['defects']['Vac@Al-Sub@Fe']['threshold']=0.0001
        mdict['defects']['Vac@Al-Sub@Fe']['charge']=[-2,-1,0,1,2,3,4,5]
        mdict['defects']['Vac@Al-Sub@Fe']['coord_type']='fractional'
        mdict['defects']['Vac@Al-Sub@Fe']['subdefect_1']=dict()
        mdict['defects']['Vac@Al-Sub@Fe']['subdefect_1']['symbol']='Al'
        mdict['defects']['Vac@Al-Sub@Fe']['subdefect_1']['type']='vacancy'
        mdict['defects']['Vac@Al-Sub@Fe']['subdefect_1']['coordinates']='blank' #np.array([0.,0.,0.],'float')
        mdict['defects']['Vac@Al-Sub@Fe']['subdefect_2']=dict()
        mdict['defects']['Vac@Al-Sub@Fe']['subdefect_2']['symbol']='Fe'
        mdict['defects']['Vac@Al-Sub@Fe']['subdefect_2']['type']='substitution'
        mdict['defects']['Vac@Al-Sub@Fe']['subdefect_2']['coordinates']='blank' #np.array([0.5,0.5,0.],'float')
        mdict['defects']['Vac@Al-Sub@Fe']['phonon']={}
        mdict['defects']['Vac@Al-Sub@Fe']['phonon']['host1']=dict()
        mdict['defects']['Vac@Al-Sub@Fe']['phonon']['host1']['phonon_center_site']='0.1 0.2 0.3'
        mdict['defects']['Vac@Al-Sub@Fe']['phonon']['host1']['phonon_center_radius']=3.0
        mdict['defects']['Vac@Al-Sub@Fe']['phonon']['host1']['threshold']=0.07
        mdict['defects']['Vac@Al-Sub@Fe']['phonon']['sub2']=dict()
        mdict['defects']['Vac@Al-Sub@Fe']['phonon']['sub2']['phonon_center_site']='0.2 0.4 0.0'
        mdict['defects']['Vac@Al-Sub@Fe']['phonon']['sub2']['phonon_center_radius']=1.0
        mdict['defects']['Vac@Al-Sub@Fe']['phonon']['sub2']['threshold']=0.1
        mdict['defects']['AntiGe@Fe']=dict()
        mdict['defects']['AntiGe@Fe']['threshold']=0.0001
        mdict['defects']['AntiGe@Fe']['charge']=[3]
        mdict['defects']['AntiGe@Fe']['coord_type']='fractional'
        mdict['defects']['AntiGe@Fe']['subdefect_1']=dict()
        mdict['defects']['AntiGe@Fe']['subdefect_1']['symbol']='Ge'
        mdict['defects']['AntiGe@Fe']['subdefect_1']['type']='antisite'
        mdict['defects']['AntiGe@Fe']['subdefect_1']['coordinates']='blank' #np.array([0.1,0.3,0.4],'float')
        mdict['defects']['AntiGe@Fe']['phonon']={}
        mdict['defects']['MgInt']=dict()
        mdict['defects']['MgInt']['threshold']=0.0001
        mdict['defects']['MgInt']['charge']=[0,1,2,3]
        mdict['defects']['MgInt']['coord_type']='fractional'
        mdict['defects']['MgInt']['subdefect_1']=dict()
        mdict['defects']['MgInt']['subdefect_1']['symbol']='Mg'
        mdict['defects']['MgInt']['subdefect_1']['type']='interstitial'
        mdict['defects']['MgInt']['subdefect_1']['coordinates']='blank' #np.array([0.,0.2,0.3],'float')
        mdict['defects']['MgInt']['phonon']={}
        maxdiff=self.maxDiff
        self.maxDiff=None
        self.assertEqual(myoptions.options['defects'],mdict) 
        self.maxDiff=maxdiff
        #self.testclass.parse_defects_section(section_name, section_content, options)

    def test_parse_recipe_section(self):
        myip = InputParser(inputfile="long_input.inp")
        minput = MASTFile("%s/recipe_lines.txt" % testdir)
        cleanlines = list()
        for line in minput.data:
            cleanlines.append(line.strip())
        myoptions = InputOptions()
        myip.parse_recipe_section('recipe',cleanlines,myoptions)
        #print myoptions
        mdict=dict()
        mdict['recipe_file'] = os.path.join(os.getenv("MAST_RECIPE_PATH"),'defects_test.txt')
        self.assertEqual(myoptions.options['recipe'],mdict)
        #self.testclass.parse_recipe_section(section_name, section_content, options)

    def test_parse_ingredients_section(self):
        myip = InputParser(inputfile="long_input.inp")
        minput = MASTFile("%s/ingredient_lines.txt" % testdir)
        cleanlines = list()
        for line in minput.data:
            cleanlines.append(line.strip())
        myoptions = InputOptions()
        myip.parse_ingredients_section('ingredients',cleanlines,myoptions)
        print myoptions
        mdict=dict()
        mdict['global']=dict()
        mdict['global']['mast_complete_method']='complete_singlerun'
        mdict['global']['mast_update_children_method']='give_structure'
        mdict['global']['mast_queue']='default'
        mdict['global']['mast_run_method']='run_singlerun'
        mdict['global']['lcharg']='False'
        mdict['global']['mast_multiplyencut']='1.5'
        mdict['global']['mast_ppn']='1'
        mdict['global']['ismear']='1'
        mdict['global']['nsw']='191'
        mdict['global']['mast_exec']='mpiexec //share/apps/vasp5.2_cNEB'
        mdict['global']['mast_nodes']='1'
        mdict['global']['mast_xc']='PW91'
        mdict['global']['prec']='Accurate'
        mdict['global']['mast_kpoints']=[2, 2, 2, 'M']
        mdict['global']['mast_write_method']='write_singlerun'
        mdict['global']['ldauu']='1 3 5'
        mdict['global']['ibrion']='2'
        mdict['global']['mast_ready_method']='ready_singlerun'
        mdict['global']['isif']='2'
        mdict['global']['mast_program']='vasp'
        mdict['global']['lwave']='False'
        mdict['global']['sigma']='0.2'
        mdict['phononingredient']=dict()
        mdict['phononingredient']['mast_exec']='phon several words WiTh CaPs'
        mdict['phononingredient']['ibrion']='5'
        mdict['phononingredient']['mast_program']='phon'
        maxdiff=self.maxDiff
        self.maxDiff=None
        self.assertEqual(myoptions.options['ingredients'],mdict)
        self.maxDiff=maxdiff
        #self.testclass.parse_ingredients_section(section_name, section_content, options)

    def test_parse_neb_section(self):
        myip = InputParser(inputfile="long_input.inp")
        minput = MASTFile("%s/neb_lines.txt" % testdir)
        cleanlines = list()
        for line in minput.data:
            cleanlines.append(line.strip())
        myoptions = InputOptions()
        myip.parse_neb_section('neb',cleanlines,myoptions)
        print myoptions
        mdict=dict()
        mdict['nebs']=dict()
        mdict['nebs']['vac1-vac2']=dict()
        mdict['nebs']['vac1-vac2']['images']=3
        mdict['nebs']['vac1-vac2']['lines']=list()
        mdict['nebs']['vac1-vac2']['phonon']=dict()
        mdict['nebs']['vac1-vac2']['lines'].append(list(['Cr',' 0.5 0.5 0.0',' 0.0 0.0 0.0']))
        mdict['nebs']['vac1-vac2']['lines'].append(list(['Fe',' 0.2 0.1 0.3',' 0.2 0.1 0.4']))
        mdict['nebs']['vac1-vac3']=dict()
        mdict['nebs']['vac1-vac3']['images']=5
        mdict['nebs']['vac1-vac3']['lines']=list()
        mdict['nebs']['vac1-vac3']['lines'].append(list(['Al',' 0.1 0.1 0.0',' 0.3 0.4 0.1']))
        mdict['nebs']['vac1-vac3']['phonon']=dict()
        mdict['nebs']['vac1-vac3']['phonon']['solvent1']=dict()
        mdict['nebs']['vac1-vac3']['phonon']['solvent1']['phonon_center_site']='0.3 0.2 0.1'
        mdict['nebs']['vac1-vac3']['phonon']['solvent1']['phonon_center_radius']=3.0
        mdict['nebs']['vac1-vac3']['phonon']['solvent1']['threshold']=0.1
        mdict['nebs']['vac1-vac3']['phonon']['solute1']=dict()
        mdict['nebs']['vac1-vac3']['phonon']['solute1']['phonon_center_site']='0.2 0.1 0.5'
        mdict['nebs']['vac1-vac3']['phonon']['solute1']['phonon_center_radius']=0.1
        mdict['nebs']['vac1-vac3']['phonon']['solute1']['threshold']=0.05
        self.assertEqual(myoptions.options['neb'],mdict)
        #self.testclass.parse_neb_section(section_name, section_content, options)

    def test_parse_chemical_potentials_section(self):
        myip = InputParser(inputfile="long_input.inp")
        minput = MASTFile("%s/chemical_potential_lines.txt" % testdir)
        cleanlines = list()
        for line in minput.data:
            cleanlines.append(line.strip())
        myoptions = InputOptions()
        myip.parse_chemical_potentials_section('chemical_potentials',cleanlines,myoptions)
        print myoptions
        mdict=dict()
        mdict['As rich']=dict()
        mdict['As rich']['As']=3.5
        mdict['As rich']['Ga']=4.5
        mdict['Ga rich']=dict()
        mdict['Ga rich']['As']=4.5
        mdict['Ga rich']['Ga']=3.5
        self.assertEqual(myoptions.options['chemical_potentials'],mdict)
        #self.testclass.parse_chemical_potentials_section(section_name, section_content, options)

    def test_parse_phonon_section(self):
        raise SkipTest
        #Phonon section in input file is now obsolete!
        myip = InputParser(inputfile="long_input.inp")
        minput = MASTFile("%s/phonon_lines.txt" % testdir)
        cleanlines = list()
        for line in minput.data:
            cleanlines.append(line.strip())
        myoptions = InputOptions()
        myip.parse_phonon_section('phonon',cleanlines,myoptions)
        print myoptions
        mdict=dict()
        mdict['perfect']=dict()
        mdict['perfect']['phonon_center_site']='0.5 0.5 0'
        mdict['perfect']['phonon_center_radius']='1'
        mdict['vac1']=dict()
        mdict['vac1']['phonon_center_site']='0.5 0.5 0'
        mdict['vac1']['phonon_center_radius']='1'
        mdict['vac2']=dict()
        mdict['vac2']['phonon_center_site']='0.0 0.0 0'
        mdict['vac2']['phonon_center_radius']='1'
        mdict['vac1-vac2']=dict()
        mdict['vac1-vac2']['phonon_center_site']='0.25 0.25 0'
        mdict['vac1-vac2']['phonon_center_radius']='1'
        self.assertEqual(myoptions.options['phonon']['phonon'],mdict)
        #self.testclass.parse_phonon_section(section_name, section_content, options)

    def test_perform_element_mapping(self):
        myip = InputParser(inputfile="element_mapping.inp")
        myoptions = myip.parse()
        self.assertItemsEqual(myoptions.options.keys(),['mast','structure','defects','recipe','ingredients','neb','chemical_potentials'])
        self.assertEqual(myoptions.options['defects']['defects']['group1']['subdefect_1']['symbol'],'Xe')
        self.assertEqual(myoptions.options['defects']['defects']['group1']['subdefect_2']['symbol'],'Ar')
        self.assertEqual(myoptions.options['defects']['defects']['group2']['subdefect_1']['symbol'],'Ar')
        self.assertEqual(myoptions.options['defects']['defects']['group2']['subdefect_2']['symbol'],'Xe')
        self.assertEqual(myoptions.options['neb']['nebs']['group1-group2']['lines'][0][0],'Ar')
        self.assertEqual(myoptions.options['neb']['nebs']['group1-group2']['lines'][1][0],'Xe')

        #self.testclass.perform_element_mapping(input_options)

    def test_validate_execs(self):
        myip = InputParser(inputfile="long_input.inp")
        myoptions = myip.parse()
        myoptions.options['ingredients']['global'].pop('mast_exec')
        myoptions.options['ingredients']['phononparse'].pop('mast_exec')
        self.assertRaises(MASTError,myip.validate_execs,myoptions)
        #self.testclass.validate_execs(input_options)

    def test_set_structure_from_inputs(self):
        myip = InputParser(inputfile="long_input.inp")
        myoptions = myip.parse()
        compare_struct = pymatgen.io.vaspio.Poscar.from_file("longinputposcar").structure
        self.assertEqual(myoptions.options['structure']['structure'],compare_struct)
        #self.testclass.set_structure_from_inputs(input_options)




##############################################################
# This code is part of the MAterials Simulation Toolkit (MAST)
# 
# Maintainer: Zhewen Song
# Last updated: 2014-04-25
##############################################################
from numpy import *
import pymatgen as mg

def getinfo(line):
    line=line.strip('\n')
    data=line.split(' ')
    while 1:
        try: data.remove('')
        except: break
    return data

def read_ele(inp):
    """Reading elements from MAST input file.
    	Args: 
    	    inp <str>: input file
    """
    fp=open(inp,'r+').readlines()
    i=0
    ele={}
    while i<len(fp):
        if 'begin elementmap' in fp[i]:
            i=i+1
            while not 'end' in fp[i]:
                line=getinfo(fp[i])
                if len(line)==2:
                    key=line[0]
                    ele[key]=line[1]
                i=i+1
        i=i+1
    return ele
            
            
def read_site(inp):
    """Reading defect sites from MAST input file.
    	Args: 
    	    inp <str>: input file
    """
    fp=open(inp,'r+').readlines()
    i=0
    site={}
    while i<len(fp):
        if '$site' in fp[i]:
            i=i+1
            while not '$end' in fp[i]:
                line=getinfo(fp[i])
                if len(line)==1:
                    key=line[0]
                    site[key]=[]
                elif len(line)==3:
                    site[key].append([float(line[0]),float(line[1]),float(line[2])]) 
                i=i+1
        i=i+1
    return site
    
def read_vec(inp):
    """Reading lattice vector from MAST input file.
    	Args: 
    	    inp <str>: input file
    """
    fp=open(inp,'r+').readlines()
    i=0           
    lattice_vectors=zeros((3,3)) 
    while i<len(fp):
        if 'begin lattice' in fp[i]:
            i=i+1; j=0
            while not 'end' in fp[i]:
                line=getinfo(fp[i])
                if len(line)==3:
                    for k in range(3):
                        lattice_vectors[j][k]=float(line[k]) 
                    j=j+1
                i=i+1     
        i=i+1
    return  lattice_vectors
    
def read_coordinates(inp):
    """Reading coordinates from MAST input file.
    	Args: 
    	    inp <str>: input file
    """
    fp=open(inp,'r+').readlines()
    i=0           
    coordinates={}   
    while i<len(fp):
        if 'begin coordinates' in fp[i]:
            i=i+1
            while not 'end' in fp[i]:
                line=getinfo(fp[i])
                if len(line)==4:
                    try: coordinates[line[0]]
                    except: coordinates[line[0]]=[]
                    coordinates[line[0]].append([float(line[1]),float(line[2]),float(line[3])])
                i=i+1
        i=i+1
    return coordinates
        
def MakeSupercell(xyz,size):
    """Making supercell with a given size.
    	Args: 
    	    xyz <dict>: the coordinates to be scaled
    	    size <list>: the scaling size
    """
    supercell={}
    for i in range(size[0]):
        for j in range(size[1]):
            for k in range(size[2]):
                for ele in xyz.keys():
                    for atom in range(len(xyz[ele])):
                        try: supercell[ele]
                        except: supercell[ele]=[]
                        new=[1.*(xyz[ele][atom][0]+i)/size[0],1.*(xyz[ele][atom][1]+j)/size[1],1.*(xyz[ele][atom][2]+k)/size[2]]
                        supercell[ele].append(new)
    return supercell
       
def distance(A,B,vec):
    """Calculating distance of two atoms considering the periodical conditions
        Args:
            A, B <list>: the two coordinates
            vec <list>: lattice vector
    """
    d=array([0.,0.,0.])
    for i in range(3):
        d[i]=A[i]-B[i]
        if d[i]>0.5: d[i]=d[i]-1
        elif d[i]<-0.5: d[i]=d[i]+1
    d=dot(d,vec)
    return double(linalg.norm(d))

def neighbors(site,Nth,vec):  
    """Get nth neighbor of possible pairs of same/different site types
        Args: 
            site <dict>: the defect sites
            Nth <int>: up to Nth neighbor
            vec <list>: lattice vector
    """
    pair={}
    for i in site.keys():
        for j in site.keys():
            val=[]
            for ii in range(len(site[i])):
                for jj in range(len(site[j])):
                    d=distance(site[i][ii],site[j][jj],vec)
                    if d>1e-5:
                        val.append((d,site[i][ii],site[j][jj]))
            dist=array(val,dtype=[('length',float),('coord1',list),('coord2',list)])  
            if len(dist)==0: return 'NaN'
            dist=sort(dist,order='length') 
            count=1
            k=0
            for k in range(len(dist)-1):
                if dist[k+1][0]-dist[k][0]>1e-3:
                    if count==Nth: 
                        if not j+'-'+i in pair.keys():
                            pair[i+'-'+j]=[dist[k][1],dist[k][2]]
                    count=count+1
            if count==Nth: 
                if not j+'-'+i in pair.keys():
                    pair[i+'-'+j]=[dist[k][1],dist[k][2]]            
            if count<Nth: return 'NaN' # Input cell too small, Nth nearest neighbor not found
    return pair            

def isline(A,M,B,vec):
    """Checking if three atoms are in a line
        Args:
            A, M, B <list>: the three coordinates
            vec <list>: lattice vector
    """
    a=distance(A,M,vec)
    b=distance(B,M,vec)
    c=distance(A,B,vec)
    p=(a+b+c)/2.0
    area=(p*(p-a)*(p-b)*(p-c))**0.5
    if c>a and c>b:
	if 2*area/c<1e-3: flag=1
        elif 2*area/c<1.0: flag=-1
	else: flag=0
    else: flag=0
    return flag
    
def iscross(xyz,vec,site,site1,site2):  
    """Checking if the path crosses over a host atom or another site
        Args:
            xyz <dict>: the host lattice coordinates
            vec <list>: lattice vector
            site <dict>: the defect sites
            site1, site2 <list>: the two ends of a path
    """
    for ele in xyz.keys():
        for i in range(len(xyz[ele])): # check if the path crosses over a host atom
            coords=xyz[ele][i]
            if not distance(coords,site1,vec)==0 and not distance(coords,site2,vec)==0:
                flag=isline(site1,coords,site2,vec)
		if not flag==0: return 'a host lattice!'
    for i in site.keys(): # check if the path crosses over another site
        for j in range(len(site[i])):
            if not distance(site[i][j],site1,vec)==0 and not distance(site[i][j],site2,vec)==0: 
                flag=isline(site1,site[i][j],site2,vec)
		if flag==1: return 'site '+i+'!'    
    return 1
            
def get_path(site,xyz,vec,N): 
    """Getting paths from the defect sites
        Args:
            site <dict>: the defect sites
            xyz <dict>: the host lattice coordinates
            vec <list>: lattice vector
    """
    path={'T':[],'F':[]}
    for nb in range(1,N+1):
        if neighbors(site,nb,vec)=='NaN':
            return 'NaN'
        else: pair=neighbors(site,nb,vec)
        for i in pair.keys():
            cross=iscross(xyz,vec,site,pair[i][0],pair[i][1])
            if cross==1:
                path['T'].append([i,nb,pair[i][0],pair[i][1]])
            else: 
                path['F'].append([i,nb,pair[i][0],pair[i][1],cross])
    return path
    

def main(inp,N):
    size=array([2,2,2])
    vec=read_vec(inp)
    xyz=read_coordinates(inp)
    site=read_site(inp)
    xyz=MakeSupercell(xyz,size)
    site=MakeSupercell(site,size)
    vec=vec*size
    path=get_path(site,xyz,vec,N) 
    i=1
    while path=='NaN':
        i=i+1
        size=size/(i-1)*i
        xyz=MakeSupercell(xyz,size)
        site=MakeSupercell(site,size)
        vec=vec*size
        path=get_path(site,xyz,vec,N)         
    if False in (size==array([1,1,1])):
        print 'The input cell size is too small to find all '+str(N)+' NN, so the size is expanded to '+str(size[0])+'x'+str(size[1])+'x'+str(size[2])+' supercell!'
    
    print 'There are '+str(len(path['T'])+len(path['F']))+' paths in total.'
    if not len(path['F'])==0:
        print 'But only '+str(len(path['T']))+' paths are possible.'
        for i in range(len(path['F'])):
            print 'Path '+path['F'][i][0]+'('+str(path['F'][i][1])+' NN) is removed because it is too close to '+path['F'][i][4]
     #   print 'These removed paths are labeled as POSCAR_F*.'
    return [vec,xyz,path]



import random
import math

def metropolis(pop, nkeep, Optimizer):
	"""Selection function that applies metropolis algorithm to determine which individuals to keep
	Inputs:
		pop = initial list of Individual class structures
		nkeep = number of structure to keep in population
			nkeep must be less than or equal to pop
		Optimizer = Optimizer class object with metropolis temperature
	Outputs:
		newpop = new list of Individual class structures of length nkeep
	"""
	newpop = []
	#Identify temperature for algorithm
	T = Optimizer.metropolis_temp
	#Collect fitnesses
	fits = [ind.fitness for ind in pop]
	#Identify the minimum fitness
	minf = min(fits)
	print fits
	while len(newpop)<nkeep:
		ind = random.choice(pop)
		r=random.random()
		if r <= math.exp((minf-ind.fitness)/T):
			newpop.append(ind)
	return newpop


"""Tests for Chopingredient"""

from MAST.ingredients.chopingredient import ChopIngredient

import unittest
from unittest import SkipTest
import os
import time
import MAST
import pymatgen
from MAST.utility import dirutil
from MAST.utility import MASTFile
import shutil

testname="chop_test_ready"
testdir = dirutil.get_test_dir(testname)

class TestIsReadyToRunIngredient(unittest.TestCase):

    def setUp(self):
        os.chdir(testdir)

        if not os.path.isdir("writedir"):
            os.mkdir("writedir")
        if not os.path.isdir("writedir/single_label1"):
            os.mkdir("writedir/single_label1")

    def tearDown(self):
        tearlist = list()
        tearlist.append("writedir")
        for foldername in tearlist:
            try:
                shutil.rmtree(foldername)
            except OSError:
                pass

    def test___init__(self):
        self.assertTrue(True)
        #raise SkipTest
        #self.testclass.__init__(**kwargs)

    def test_ready_singlerun(self):
        ingdir="%s/writedir/single_label1" % testdir
        recipedir="%s/writedir" % testdir
        topmetad = MASTFile("files/top_metadata_single")
        topmetad.data.append("origin_dir = %s/files\n" % testdir) #give origin directory
        topmetad.to_file("writedir/metadata.txt")
        metad = MASTFile("files/metadata_single")
        metad.to_file("%s/metadata.txt" % ingdir)
        kdict=dict()
        kdict['mast_program'] = 'vasp'
        kdict['mast_kpoints'] = [2,2,2,"M"]
        kdict['mast_xc'] = 'pw91'
        my_structure = pymatgen.io.vaspio.Poscar.from_file("files/perfect_structure").structure
        mywr = ChopIngredient(name=ingdir, program_keys = kdict, structure=my_structure)
        mywr.write_singlerun()
        myrdi = ChopIngredient(name=ingdir,program_keys=kdict, structure=my_structure)
        self.assertTrue(myrdi.ready_singlerun())
        os.remove("%s/POSCAR" % ingdir)
        self.assertFalse(myrdi.ready_singlerun())
        #self.testclass.ready_singlerun()

    def test_ready_structure(self):
        ingdir="%s/writedir/single_label1" % testdir
        recipedir="%s/writedir" % testdir
        topmetad = MASTFile("files/top_metadata_single")
        topmetad.data.append("origin_dir = %s/files\n" % testdir) #give origin directory
        topmetad.to_file("writedir/metadata.txt")
        metad = MASTFile("files/metadata_single")
        metad.to_file("%s/metadata.txt" % ingdir)
        kdict=dict()
        kdict['mast_program'] = 'vasp'
        kdict['mast_kpoints'] = [2,2,2,"M"]
        kdict['mast_xc'] = 'pw91'
        my_pos = pymatgen.io.vaspio.Poscar.from_file("files/perfect_structure")
        my_pos.write_file("writedir/single_label1/POSCAR")
        myrdi = ChopIngredient(name=ingdir,program_keys=kdict, structure=my_pos.structure)
        self.assertTrue(myrdi.ready_structure())
        os.remove("%s/POSCAR" % ingdir)
        self.assertFalse(myrdi.ready_structure())
        #self.testclass.ready_structure()

    def test_ready_neb_subfolders(self):
        ingdir="%s/writedir/single_label1" % testdir
        recipedir="%s/writedir" % testdir
        topmetad = MASTFile("files/top_metadata_single")
        topmetad.data.append("origin_dir = %s/files\n" % testdir) #give origin directory
        topmetad.to_file("writedir/metadata.txt")
        metad = MASTFile("files/metadata_single")
        metad.to_file("%s/metadata.txt" % ingdir)
        kdict=dict()
        kdict['mast_program'] = 'vasp'
        kdict['mast_kpoints'] = [2,2,2,"M"]
        kdict['mast_xc'] = 'pw91'
        kdict['mast_neb_settings']=dict()
        kdict['mast_neb_settings']['images'] = 3
        my_structure = pymatgen.io.vaspio.Poscar.from_file("files/perfect_structure").structure
        mywr = ChopIngredient(name=ingdir, program_keys = kdict, structure=my_structure)
        for subdir in ['00','01','02','03','04']:
            subname = "%s/%s" % (ingdir, subdir)
            os.mkdir(subname)
            mywr.keywords['name'] = subname
            mywr.checker.keywords['name'] = subname 
            if not subdir in ['00','04']:
                mywr.write_singlerun()
                mywr.write_submit_script()
        myrdi = ChopIngredient(name=ingdir,program_keys=kdict, structure=my_structure)
        self.assertTrue(myrdi.ready_neb_subfolders())
        os.remove("%s/01/POSCAR" % ingdir)
        self.assertFalse(myrdi.ready_neb_subfolders())
        #self.testclass.ready_neb_subfolders()

    def test_ready_subfolders(self):
        ingdir="%s/writedir/single_label1" % testdir
        recipedir="%s/writedir" % testdir
        topmetad = MASTFile("files/top_metadata_single")
        topmetad.data.append("origin_dir = %s/files\n" % testdir) #give origin directory
        topmetad.to_file("writedir/metadata.txt")
        metad = MASTFile("files/metadata_single")
        metad.to_file("%s/metadata.txt" % ingdir)
        kdict=dict()
        kdict['mast_program'] = 'vasp'
        kdict['mast_kpoints'] = [2,2,2,"M"]
        kdict['mast_xc'] = 'pw91'
        kdict['images'] = 3
        my_structure = pymatgen.io.vaspio.Poscar.from_file("files/perfect_structure").structure
        mywr = ChopIngredient(name=ingdir, program_keys = kdict, structure=my_structure)
        for subdir in ['00','01','02','03','04']:
            subname = "%s/%s" % (ingdir, subdir)
            os.mkdir(subname)
            mywr.keywords['name'] = subname
            mywr.checker.keywords['name'] = subname 
            mywr.write_singlerun()
            mywr.write_submit_script()
        myrdi = ChopIngredient(name=ingdir,program_keys=kdict, structure=my_structure)
        self.assertTrue(myrdi.ready_subfolders())
        os.remove("%s/00/POSCAR" % ingdir)
        self.assertFalse(myrdi.ready_subfolders())
        #self.testclass.ready_subfolders()




#!/usr/bin/env python
##############################################################
# This code is part of the MAterials Simulation Toolkit (MAST)
# This procar code should be replaced with pymatgen functions. 
# Maintainer: Tam Mayeshiba
# Last updated: 2014-04-25
##############################################################

"""
Classes for reading/manipulating/writing VASP ouput files.
"""

from __future__ import division

__author__ = "Tam Mayeshiba from vasp_output by Shyue Ping Ong"
__date__ = "April 8, 2012"

import os
import glob
import re
import math
import itertools
import warnings
import xml.sax.handler
import StringIO
from collections import defaultdict
import logging

import numpy as np

from pymatgen.util.coord_utils import get_points_in_sphere_pbc
from pymatgen.util.io_utils import zopen, clean_lines, micro_pyawk, \
    clean_json, reverse_readline
from pymatgen.core.structure import Structure
from pymatgen.core.composition import Composition
from pymatgen.electronic_structure.core import Spin, Orbital
from pymatgen.electronic_structure.dos import CompleteDos, Dos
from pymatgen.electronic_structure.bandstructure import BandStructure, \
    BandStructureSymmLine, get_reconstructed_band_structure
from pymatgen.core.lattice import Lattice
from pymatgen.io.vaspio.vasp_input import Incar, Kpoints, Poscar

logger = logging.getLogger(__name__)




class ProcarOcc(object):
    """
    Object for reading a PROCAR file
    """
    def __init__(self, filename):
        """
        Args:
            filename:
                Name of file containing PROCAR.
        """
        #create and return data object containing the information of a PROCAR
        self.name = ""
        self.occonly = {}
        self.allbands= {}
        self.occonly['up'] = dict()
        self.occonly['down'] = dict()
        self.allbands['up'] = dict()
        self.allbands['down'] = dict()
        self.headers = None
        myocc=0
        with zopen(filename, "r") as f:
            #lines = list(clean_lines(f.readlines())) #TTM do not clean lines
            lines = list(f.readlines())
            self.name = lines[0]
            kpointexpr = re.compile("^\s*k-point\s+(\d+).*weight = ([0-9\.]+)")
            ionexpr = re.compile("^ion.*")
            expr = re.compile("^\s*([0-9]+)\s+")
            dataexpr = re.compile("[\.0-9]+")
            weight = 0
            foundtimes = 0
            usekey=""

            for l in lines:
                if l.find("occ.")>-1:
                    myocc=float(l.split()[7])
                if kpointexpr.match(l):
                    m = kpointexpr.match(l)
                    currentKpoint = int(m.group(1))
                    weight = float(m.group(2))
                    if currentKpoint == 1:
                        foundtimes = foundtimes + 1
                    if foundtimes == 1:
                        usekey = 'up'
                    elif foundtimes == 2:
                        usekey = 'down'
                    else:
                        print "Error: foundtimes > 2!"
                        return
                elif ionexpr.match(l) and self.headers is None:
                    self.headers = l.split()
                    self.headers.pop(0)
                elif expr.match(l):
                    linedata = dataexpr.findall(l)
                    linefloatdata = map(float, linedata)
                    index = int(linefloatdata.pop(0))
                    if index in self.occonly[usekey]:
                        if myocc==1:
                            self.occonly[usekey][index] += np.array(linefloatdata) * weight
                        self.allbands[usekey][index] += np.array(linefloatdata) * weight
                    else:
                        if myocc==1:
                            self.occonly[usekey][index] = np.array(linefloatdata) * weight
                        else:
                            self.occonly[usekey][index]=0.0
                        self.allbands[usekey][index] = np.array(linefloatdata) * weight

    def get_d_occupation(self, atom_index):
        """
        .. deprecated:: v2.6.4

            Use get_occpuation instead.

        Returns the d occupation of a particular atom.

        Args:
            atom_index:
                Index of atom in PROCAR.

        Returns:
            d-occupation of atom at atom_index.
        """
        warnings.warn("get_d_occupation has been deprecated. Use "
                      "get_occupation instead.", DeprecationWarning)
        return self.get_occupation(atom_index, 'd')

    def get_occupation(self, atom_index, orbital):
        """
        Returns the occupation for a particular orbital of a particular atom.

        Args:
            atom_num:
                Index of atom in the PROCAR
            orbital:
                A string representing an orbital. If it is a single
                character, e.g., s, p, d or f, the sum of all s-type,
                p-type, d-type or f-type orbitals occupations are returned
                respectively. If it is a specific orbital, e.g., px, dxy,
                etc., only the occupation of that orbital is returned.

        Returns:
            Sum occupation of orbital of atom.
        """
        totalup = 0.0
        totaldown = 0.0
        totaloccup = 0.0
        totaloccdown = 0.0
        for mykey in ['up','down']:
            row_occonly = self.occonly[mykey][atom_index]
            row = self.allbands[mykey][atom_index]
            #print "OCCONLY: ",row_occonly
            #print "ALLBANDS:", row
            total_occonly = 0.0
            found_occonly = False
            total = 0.0
            found = False
            for orb, data in zip(self.headers, row_occonly):
                #print "ORB_occonly: ",orb
                #print "DATA_occonly:",data
                if orb.startswith(orbital):
                    found_occonly = True
                    total_occonly += data
            if not found_occonly:
                raise ValueError("Occonly Invalid orbital {}".format(orbital))
            for orb, data in zip(self.headers, row):
                #print "ORB_allbands: ",orb
                #print "DATA_allbands:",data
                if orb.startswith(orbital):
                    found = True
                    total += data
            if not found:
                raise ValueError("Allbands Invalid orbital {}".format(orbital))
            if mykey == 'up':
                totalup = total
                totaloccup = total_occonly
            else:
                totaldown = total
                totaloccdown = total_occonly
        return [totalup, totaldown, totalup + totaldown, totaloccup, totaloccdown, totaloccup + totaloccdown]





from ase import Atom, Atoms
import random

def generate_dumbbells(ndumbbells,dumbbellsym,nindiv,solid,symbol=None,size=None):
    """Function to generate initial dumbbells 
    Inputs:
        ndumbbells = number of dumbbells to generate
        dumbbellsym = symbol of atom for dumbell
        nidiv = number of individuals to generate
        solid = ASE Atoms class object with initial bulk structure for dumbbells to reside
        symbol = dumbbell symbol matching
        size = restrict location of dumbbells
    Outputs:
        indivs = list of ASE atoms class objects with dumbbells
    *** Note: function is still in development.  Has not been implemented ***
    """
    if size==None:
        size = ndumbbells**0.333*3.0
    bulk = solid.copy()
    bulkcom = bulk.get_center_of_mass()
    bulk.translate(-bulkcom)
    bulk.append(Atom(position=[0,0,0]))
    nr2 = Atoms(pbc=True, cell=bulk.get_cell())
    for i in range(len(bulk)-1):
        dist = bulk.get_distance(-1,i)
        if dist <= size:
            nr2.append(bulk[i])
    indivs = []
    for one in range(nindiv):
        indices = []
        if symbol == None:
            opts = [atm for atm in nr2]
        else:
            opts = [atm for atm in nr2 if atm.symbol==symbol]
        indiv = Atoms()
        if len(opts) <= ndumbbells:
            ndum = len(opts)
            flag = True
        else:
            ndum = ndumbbells
            flag = False
        for one in range(ndum):
            while True:
                atm = random.choice(opts)
                if atm.index not in indices:
                    break
            datm = Atom(symbol=dumbbellsym, position=atm.position)
            indiv.append(datm)
        if flag==True:
            lack = ndumbbells-ndum
            for one in range(lack):
                indiv.append(Atom(symbol=dumbbellsym,position=random.choice(opts).position))
        indiv.translate(bulkcom)
        indiv.rattle(stdev=0.5)
        indivs.append(indiv)
    return indivs


from MAST.structopt.tools import rattle
import logging
try:
    from mpi4py import MPI
except ImportError:
    pass
import pdb

def fitness_switch(input):
    """Performs fitness evaluation on individual"""
    if input[0]==None:
        indiv=0
        rank = MPI.COMM_WORLD.Get_rank()
        stro='Evaluated none individual on {0}\n'.format(rank)
    else:
        Optimizer,indiv=input
        #logger = initialize_logger(Optimizer.loggername)
        logger = logging.getLogger(Optimizer.loggername)
        stro='Evaluating individual {0}\n'.format(indiv.index)
        if Optimizer.rattle_atoms:
            indiv[0]=rattle(indiv)
        scheme = Optimizer.fitness_scheme
        strn = ''
        try:
            exec "from MAST.structopt.fitness.{0} import {0}".format(scheme)
            indiv, strn = eval('{0}(indiv, Optimizer)'.format(scheme))
        except NameError, e:
            print e
            print 'WARNING: Specified fitness not one of the available options. Please check documentation and spelling! Fitness : '+repr(scheme)
            logger.warn('NameError in fitness switch for scheme = {0}. {1}'.format(scheme,e), exc_info=True)
        except Exception, e:
            print 'Error in Fitness fuction : '+repr(scheme)
            print e
            logger.warn('Exception in fitness switch for {0} scheme: {1}'.format(scheme,e),exc_info=True)
        stro+=strn
    return indiv,stro

# if __name__ == "__main__":
#     import sys
#     indiv = sys.argv[1]
#     Optimizer = sys.argv[2]
#     output = fitness_switch([Optimizer,indiv])
#     output[0].write()
#     Optimizer.output.write(output[1])



import os
import copy
from MAST.structopt.tools.lammps import LAMMPS

def cell_relax_lammps(indiv, Optimizer):
    """Move function to perform Lammps box/relax for cell. Intended for use in Crystal Optimization
    Inputs:
        indiv = Individual class object to be altered
        Optimizer = Optimizer class object with needed parameters
    Outputs:
        indiv = Altered Individual class object
    """
    if 'MU' in Optimizer.debug:
        debug = True
    else:
        debug = False
    cwd1=os.getcwd()
    Optimizer.output.write('LAMMPS fix box/relax performed\n')
    parameters=copy.deepcopy(Optimizer.calc.parameters)
    passflag=True
    try:
        parameters['mass'][len(parameters['mass'])-1] += '\nfix 1 all box/relax iso 0.0 vmax 0.001'
    except KeyError:
        try:
            parameters['pair_coeff'][0] += '\nfix 1 all box/relax iso 0.0 vmax 0.001'
        except KeyError:
            print 'WARNING: LAMMPS Cell relax move trouble with potential. SKIPPING'
            Optimizer.output.write('WARNING: Minimizer quit.  LAMMPS Cell relax move trouble with potential - Box relaxation unsuccessful\n')
            passflag=False
    if passflag:
        filesL = [ Optimizer.pot_file ]
        if Optimizer.lammps_keep_files:
            calc2 = LAMMPS(parameters=parameters, files=filesL,keep_tmp_files=True, tmp_dir=os.getcwd()+'/'+Optimizer.filename+'/Lammpsrun2/')
        else:
            calc2 = LAMMPS(parameters=parameters, files=filesL)
        atmsdup=indiv[0].copy()
        atmsdup.set_calculator(calc2)
        try:
            OUT=atmsdup.calc.calculate(atmsdup)
            indiv[0].set_cell(OUT['atoms'].get_cell())
            indiv[0].set_positions(OUT['atoms'].get_positions())
            Optimizer.output.write('Energy = '+repr(OUT['thermo'][-1]['pe']/indiv[0].get_number_of_atoms())+'\n')
        except:
            Optimizer.output.write('WARNING: Minimizer quit - Box relaxation unsuccessful\n')
        #pdb.set_trace()
        os.chdir(cwd1)
        calc2.clean()
        Optimizer.output.write(repr(indiv[0].get_cell())+'\n')
        muttype='LBR'
        if indiv.energy==0:
            indiv.history_index=indiv.history_index+'m'+muttype
        else:
            indiv.history_index=repr(indiv.index)+'m'+muttype
    return indiv


"""Tests for Recipeutility"""

#from MAST.recipe.recipeutility import RecipeUtility

import unittest
from unittest import SkipTest
import os
import time
import pymatgen
from MAST.utility import dirutil
from MAST.recipe import recipeutility
testname="recipeutility_test"
testdir = dirutil.get_test_dir(testname)

class TestRecipeUtility(unittest.TestCase):

    def setUp(self):
        os.chdir(testdir)

    def tearDown(self):
        pass

    def test_read_recipe(self):
        [htu, ptc,htr, rname]=recipeutility.read_recipe(os.path.join(testdir,'short_test'))
        #print htu
        #print ptc
        #print htr
        #print rname
        self.assertEqual(htu['neb_group1-group3_q=n1_opt2']['neb_group1-group3_q=n1_stat'],'neb_to_nebstat')
        self.assertItemsEqual(ptc['neb_group1-group3_q=n1_opt1'],['defect_group1_q=n1_stat','defect_group3_q=n1_stat'])
        self.assertEqual(htr['neb_group1-group3_q=n1_stat'],'nebstat_to_nebphonon')
        #self.testclass.read_recipe(filename, verbose=0)

    def test_split_into_subrecipes(self):
        raise SkipTest
        #self.testclass.split_into_subrecipes(mydata)

    def test_get_indent_level(self):
        raise SkipTest
        #self.testclass.get_indent_level(myline)

    def test_parse_for_name_and_instructions(self):
        raise SkipTest
        #self.testclass.parse_for_name_and_instructions(myline)

    def test_parse_indentation_dict(self):
        raise SkipTest
        #self.testclass.parse_indentation_dict(idict)

    def test_make_indentation_dictionary(self):
        raise SkipTest
        #self.testclass.make_indentation_dictionary(subrlist)




#!/bin/env python
##############################################################
# This code is part of the MAterials Simulation Toolkit (MAST)
# 
# Maintainer: Tam Mayeshiba
# Last updated: 2014-04-25
##############################################################
#TTM 13Mar2014
def doc():
    """This routine numbers all the lines in a text file
        and produces a new text file.
        e.g. 
            apple
            banana
            carrot
        becomes
            1.apple
            2.banana
            3.carrot
        Call as: python number_me.py original_file file_to_produce
        
        python number_me.py unnumbered numbered
    """
import os
import sys

if len(sys.argv) < 3:
    print "Not enough inputs."
    print doc.__doc__
    sys.exit(-1)

myname = sys.argv[1]
if not os.path.isfile(myname):
    print "No such file %s" % myname
    sys.exit(-1)
newname = sys.argv[2]
if os.path.isfile(newname):
    print "Cannot overwrite existing file %s" % newname
    sys.exit(-1)

openfile = open(myname, 'rb')
mylines = openfile.readlines()
openfile.close()
lct=0
numberedlines=list()
for myline in mylines:
    lct = lct + 1
    numline = "%i.%s" % (lct, myline)
    numberedlines.append(numline)
newfile = open(newname, 'wb')
newfile.writelines(numberedlines)
newfile.close()
print "Done! Check output in %s" % newname



#!/usr/bin/env python
##############################################################
# This code is part of the MAterials Simulation Toolkit (MAST)
# 
# Maintainer: Tam Mayeshiba
# Last updated: 2014-04-25
##############################################################
from MAST.controllers.mastmon import MASTmon

mastmon = MASTmon()
#mastmon.run(1) #Use 0 for non-verbose



from ase import Atom, Atoms

def write_xyz(fileobj,atms,data=0):
    """Function to write xyz file with some data
    adapted from ase.io.xyz"""
    if isinstance(fileobj, str):
        fileobj=open(fileobj, 'a')
    symbols = atms.get_chemical_symbols()
    natoms = len(symbols)
    fileobj.write('%d\n' % natoms)
    fileobj.write(repr(data)+'\n')
    for s, (x, y, z) in zip(symbols, atms.get_positions()):
        fileobj.write('%-2s %22.15f %22.15f %22.15f\n' % (s, x, y, z))
    return



#TTM from distribute_setup import use_setuptools
#TTM use_setuptools()
from setuptools.command.install import install
from setuptools import setup, find_packages
from distutils.command.build_py import build_py as _build_py

#python version check
import sys
import os
if sys.version_info[0] < 2 or (sys.version_info[0] == 2 and sys.version_info[1] < 7) or (sys.version_info[0] == 2 and sys.version_info[1] == 7 and sys.version_info[2] < 3):
    print "Python Version %d.%d.%d found" % (sys.version_info[0], sys.version_info[1], sys.version_info[2])
    print "Python version >= 2.7.3 needed!!!"
    sys.exit(0)

myhome = os.getenv("HOME")
def output_env_variable_info():
    print ""
    print ""
    print ""
    print "==============================================="
    print "*************** ATTENTION *********************"
    print "Add the following lines to your //home/user/.bashrc file"
    print "or a similar configuration file."
    print "Then, log out and log back in."
    print "See the MAST manual for more information."
    print "==============================================="
    print ""
    print "export MAST_RECIPE_PATH=%s/MAST/recipe_templates" % myhome
    print "export MAST_SCRATCH=%s/MAST/SCRATCH" % myhome
    print "export MAST_ARCHIVE=%s/MAST/ARCHIVE" % myhome
    print "export MAST_CONTROL=%s/MAST/CONTROL" % myhome
    platform_list=["aci",
                    "bardeen",
                    "dlx",
                    "korczak",
                    "no_queue_system",
                    "pbs_generic",
                    "sge_generic",
                    "slurm_generic",
                    "stampede",
                    "turnbull"]
    myplatforms = "|".join(platform_list)
    print "export MAST_PLATFORM=<choose one of: %s>" % myplatforms
    print "==============================================="
    print ""
    print ""
    return
    
class build_py(_build_py):
    """Specialized Python source builder."""
    print "Starting setup for the MAterials Simulation Toolkit."
    output_env_variable_info()

setup(
        name="MAST",
        packages=find_packages(),
        version="1.1.1_beta",
        #setup_requires=["numpy>=1.6.1"],
        install_requires=["numpy>=1.6.1", "scipy>=0.10.1", "pymatgen>=2.8.8", "custodian>=0.5.1"],
        scripts=["MAST/bin/mast",
                "MAST/bin/mast_diffusion_coefficient",
                "MAST/bin/mast_defect_formation_energy"],
        data_files=[
            ("%s/MAST/recipe_templates" % myhome,
                ["MAST/recipe_templates/neb_with_phonons.txt", 
                "MAST/recipe_templates/simple_optimization.txt",
                "MAST/recipe_templates/u_ramping.txt",
                "MAST/recipe_templates/defect_formation_energy.txt"]),
            ("%s/MAST/examples" % myhome,
                ["MAST/examples/README",
                "MAST/examples/neb_with_phonons.inp",
                "MAST/examples/simple_optimization.inp",
                "MAST/examples/u_ramping.inp",
                "MAST/examples/defect_formation_energy.inp",
                "MAST/examples/POSCAR.ga4as4"]),
            ("%s/MAST/SCRATCH" % myhome,
                ["MAST/initialization/README_Scratch"]),
            ("%s/MAST/ARCHIVE" % myhome,
                ["MAST/initialization/README_Archive"]),
            ("%s/MAST/CONTROL" % myhome,
                ["MAST/initialization/README_Control",
                "MAST/initialization/submitlist",
                "MAST/initialization/just_submitted",
                "MAST/submit/runmast.py"]),
            ("%s/MAST/CONTROL/platforms/aci" % myhome,
                ["MAST/submit/platforms/aci/mastmon_submit.sh",
                "MAST/submit/platforms/aci/submit_template.sh"]),
            ("%s/MAST/CONTROL/platforms/bardeen" % myhome,
                ["MAST/submit/platforms/bardeen/mastmon_submit.sh",
                "MAST/submit/platforms/bardeen/submit_template.sh"]),
            ("%s/MAST/CONTROL/platforms/dlx" % myhome,
                ["MAST/submit/platforms/dlx/mastmon_submit.sh",
                "MAST/submit/platforms/dlx/submit_template.sh"]),
            ("%s/MAST/CONTROL/platforms/korczak" % myhome,
                ["MAST/submit/platforms/korczak/mastmon_submit.sh",
                "MAST/submit/platforms/korczak/submit_template.sh"]),
            ("%s/MAST/CONTROL/platforms/no_queue_system" % myhome,
                ["MAST/submit/platforms/no_queue_system/mastmon_submit.sh",
                "MAST/submit/platforms/no_queue_system/submit_template.sh"]),
            ("%s/MAST/CONTROL/platforms/pbs_generic" % myhome,
                ["MAST/submit/platforms/pbs_generic/mastmon_submit.sh",
                "MAST/submit/platforms/pbs_generic/submit_template.sh"]),
            ("%s/MAST/CONTROL/platforms/sge_generic" % myhome,
                ["MAST/submit/platforms/sge_generic/mastmon_submit.sh",
                "MAST/submit/platforms/sge_generic/submit_template.sh"]),
            ("%s/MAST/CONTROL/platforms/slurm_generic" % myhome,
                ["MAST/submit/platforms/slurm_generic/mastmon_submit.sh",
                "MAST/submit/platforms/slurm_generic/submit_template.sh"]),
            ("%s/MAST/CONTROL/platforms/stampede" % myhome,
                ["MAST/submit/platforms/stampede/mastmon_submit.sh",
                "MAST/submit/platforms/stampede/submit_template.sh"]),
            ("%s/MAST/CONTROL/platforms/turnbull" % myhome,
                ["MAST/submit/platforms/turnbull/mastmon_submit.sh",
                "MAST/submit/platforms/turnbull/submit_template.sh"]),
            ("%s/MAST/CONTROL/citations" % myhome,
                ["MAST/summary/citations/mast_01",
                "MAST/summary/citations/mast_02",
                "MAST/summary/citations/pymatgen_01",
                "MAST/summary/citations/spglib_01",
                "MAST/summary/citations/structopt_01",
                "MAST/summary/citations/vasp_01",
                "MAST/summary/citations/vasp_02",
                "MAST/summary/citations/vasp_03",
                "MAST/summary/citations/vasp_04",
                "MAST/summary/citations/vaspneb_01",
                "MAST/summary/citations/vaspneb_02",
                "MAST/summary/citations/vaspneb_03",
                "MAST/summary/citations/vaspneb_04",
                "MAST/summary/citations/vaspneb_05",
                "MAST/summary/citations/vaspneb_06",
                "MAST/summary/citations/vasp_paw_01",
                "MAST/summary/citations/vasp_pps_01"]),
            ("%s/MAST/CONTROL/programkeys" % myhome,
                ["MAST/ingredients/programkeys/diff_allowed_keywords.py",
                "MAST/ingredients/programkeys/structopt_allowed_keywords.py",
                "MAST/ingredients/programkeys/vasp_allowed_keywords.py"]),
            ("%s/MAST/CONTROL" % myhome,
                ["MAST/structopt/Optimizer.py"])
        ],

        author="MAST Development Team, University of Wisconsin-Madison Computational Materials Group",
        author_email="ddmorgan@wisc.edu",
        #maintainer="Tam Mayeshiba",
        url="https://materialshub.org",
        license="MIT",
        description="MAterials Simulation Toolkit",
        long_description="MAterials Simulation Toolkit for diffusion and defects",
        keywords=["MAST","materials","simulation","diffusion","defects","ab initio","high throughput", "DFT", "density functional theory", "defect formation"],
        cmdclass={'build_py': build_py}
)



import random

def tournament2(pop, nkeep, Optimizer):
    """Selection function to order and choose individuals based on Tournament schemes
    Function will allow repeats but not back to back
    """
    tournsize = Optimizer.tournsize
    newpop = []
    prevind = None
    while len(newpop) < nkeep:
        subgroup = []
        for i in range(tournsize):
            subgroup.append(random.choice(pop))
        fitnesses = [ind.fitness for ind in subgroup]
        mine = min(fitnesses)
        select = [ind for ind in subgroup if ind.fitness==mine]
        if select[0].index != prevind:
            newpop.append(select[0])
            prevind = select[0].index
    return newpop


##############################################################
# This code is part of the MAterials Simulation Toolkit (MAST)
# 
# Maintainer: Tam Mayeshiba
# Last updated: 2014-04-25
##############################################################
import os
import subprocess
import time
import logging
from MAST.utility import MASTObj
from MAST.utility import MASTError
from MAST.utility import dirutil
from MAST.utility import Metadata
from MAST.utility import MASTFile
from MAST.utility import loggerutils
from MAST.ingredients.checker import BaseChecker
from MAST.ingredients.checker import VaspChecker
from MAST.ingredients.checker import VaspNEBChecker
from MAST.ingredients.checker import PhonChecker
from MAST.ingredients.checker import GenericChecker
from MAST.ingredients.checker import LammpsChecker
from MAST.ingredients.checker import StructoptChecker
from MAST.ingredients.errorhandler import BaseError
from MAST.ingredients.errorhandler import VaspError
from MAST.ingredients.errorhandler import PhonError
from MAST.ingredients.errorhandler import VaspNEBError
from MAST.ingredients.errorhandler import GenericError
from MAST.utility import loggerutils

class BaseIngredient(MASTObj):
    """Base Ingredient class
        Attributes:
            self.meta_dict <dict>: Metadata dictionary
            self.metafile <Metadata>: Metadata file
            self.program <str>: program name, all lowercase,
                                from 'mast_program' in input
                                file
            self.checker <VaspChecker, PhonChecker, etc.>:
                    program-dependent checker object
            self.errhandler <VaspError, PhonError, etc.>:
                    program-dependent handler object
    """
    def __init__(self, allowed_keys, **kwargs):
        allowed_keys_base = dict()
        allowed_keys_base.update(allowed_keys) 
        MASTObj.__init__(self, allowed_keys_base, **kwargs)

        work_dir = '/'.join(self.keywords['name'].split('/')[:-1])
        topmeta = Metadata(metafile='%s/metadata.txt' % work_dir)
        data = topmeta.read_data(self.keywords['name'].split('/')[-1])

        self.meta_dict = dict()
        if data:
            for datum in data.split(','):
                self.meta_dict[datum.split(':')[0]] = datum.split(':')[1].strip()

        self.metafile = Metadata(metafile='%s/metadata.txt' % self.keywords['name'])

        self.program = self.keywords['program_keys']['mast_program'].lower()
        
        self.logger = logging.getLogger(self.keywords['name'])
        self.logger = loggerutils.add_handler_for_recipe(self.keywords['name'], self.logger)
        
        if self.program == 'vasp':
            self.checker = VaspChecker(name=self.keywords['name'],
            program_keys = self.keywords['program_keys'],
            structure = self.keywords['structure'])
            self.errhandler = VaspError(name=self.keywords['name'],
            program_keys = self.keywords['program_keys'],
            structure = self.keywords['structure'])
        elif self.program == 'vasp_neb':
            self.checker = VaspNEBChecker(name=self.keywords['name'],
            program_keys = self.keywords['program_keys'],
            structure = self.keywords['structure'])
            self.errhandler = VaspNEBError(name=self.keywords['name'],
            program_keys = self.keywords['program_keys'],
            structure = self.keywords['structure'])
        elif self.program == 'phon':
            self.checker = PhonChecker(name=self.keywords['name'],program_keys=self.keywords['program_keys'],structure=self.keywords['structure'])
            self.errhandler = PhonError(name=self.keywords['name'],program_keys=self.keywords['program_keys'],structure=self.keywords['structure'])
        elif self.program == 'lammps':
            self.checker = LammpsChecker(name=self.keywords['name'],program_keys=self.keywords['program_keys'],structure=self.keywords['structure'])
            self.errhandler = GenericError(name=self.keywords['name'],program_keys=self.keywords['program_keys'],structure=self.keywords['structure'])
        elif self.program =='structopt':
            self.checker = StructoptChecker(name=self.keywords['name'],program_keys=self.keywords['program_keys'],structure=self.keywords['structure'])
            self.errhandler = GenericError(name=self.keywords['name'],program_keys=self.keywords['program_keys'],structure=self.keywords['structure'])
        else:
            allowed_keys={'name','program_keys','structure'}
            self.checker = GenericChecker(name=self.keywords['name'],program_keys=self.keywords['program_keys'],structure=self.keywords['structure'])
            self.errhandler = GenericError(name=self.keywords['name'],program_keys=self.keywords['program_keys'],structure=self.keywords['structure'])

    def write_directory(self):
        try:
            os.makedirs(self.keywords['name'])
            self.metafile.write_data('directory created', time.asctime())
            self.metafile.write_data('name', self.keywords['name'].split('/')[-1])
            self.metafile.write_data('program', self.program)
            self.metafile.write_data('ingredient type', self.__class__.__name__)
            #if 'mast_charge' in self.keywords['program_keys']:
            #    self.metafile.write_data('charge', self.keywords['program_keys']['mast_charge'])
            for key, value in self.meta_dict.items():
                self.metafile.write_data(key, value)
        except OSError:
            self.logger.info("Directory for %s already exists." % self.keywords['name'])
            return
        return

    def is_complete(self):
        '''Function to check if Ingredient is ready'''
        if not self.checker.is_started():
            return False #hasn't started running yet
        complete = self.checker.is_complete()
        frozen = self.checker.is_frozen()
        if complete or frozen:
            errct = self.errhandler.loop_through_errors()
            if errct > 0:
                if os.path.isfile(os.path.join(self.keywords['name'],'error.5.tar.gz')):
                    self.logger.error("Ingredient directory already has 5 error zip files. A manual look is required.")
                    self.change_my_status("E")
                    return False
                if 'mast_auto_correct' in self.keywords['program_keys'].keys():
                    if str(self.keywords['program_keys']['mast_auto_correct']).strip()[0].lower() == 'f':
                        self.change_my_status("E")
                    else:
                        self.change_my_status("S")
                else:
                    self.change_my_status("S")
                self.errhandler.clean_up_directory()
                return False
            else:
                if complete:
                    self.metafile.write_data('completed on', time.asctime())
                    if 'get_energy_from_energy_file' in dirutil.list_methods(self.checker,0):
                        energy = self.checker.get_energy_from_energy_file()
                        self.metafile.write_data('energy', energy)
                    return True
                else:
                    return False
        else:
            return False

    def directory_is_locked(self):
        return dirutil.directory_is_locked(self.keywords['name'])

    def lock_directory(self):
        return dirutil.lock_directory(self.keywords['name'])

    def unlock_directory(self):
        return dirutil.unlock_directory(self.keywords['name'])

    def wait_to_write(self):
        return dirutil.wait_to_write(self.keywords['name'])
   
    def is_ready_to_run(self):
        if self.directory_is_locked():
            return False
        return self.checker.is_ready_to_run()
        
    def getpath(self):
        '''getpath returns the directory of the ingredient'''
        return self.keywords['name']
    
    def write_submit_script(self):
        from MAST.submit import script_commands
        script_commands.write_submit_script(self.keywords)
        return
    
    def run(self, mode='serial', curdir=os.getcwd()):
        from MAST.submit import queue_commands 
        if mode.lower() == 'noqsub':
            curdir = os.getcwd()
            os.chdir(self.keywords['name'])
            programpath = queue_commands.direct_shell_command()
            p = subprocess.Popen(programpath, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
            p.wait()
            os.chdir(curdir)
            self.metafile.write_data('run', time.asctime())
        elif mode.lower() == 'serial':
            queuesub = queue_commands.write_to_submit_list(self.keywords['name'])
            #runme = subprocess.Popen(queuesub, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
            #runme.wait()
            # for scheduling other jobs
            #runme.wait()
            self.metafile.write_data('queued', time.asctime())
        return

    def get_name(self):
        return self.keywords['name'].split('/')[-1]

    @property
    def name(self):
        return self.get_name()

    def get_keywords(self):
        return self.keywords.copy()

    def __repr__(self):
        return 'Ingredient %s of type %s' % (self.keywords['name'].split('/')[-1], self.__class__.__name__)

    def get_my_label(self, label):
        """Get the value of a label in the metadata file.
            Args:
                label <str>: Label to search for.
            Returns:
                <str>: Value of the label as a string, stripped.
        """
        myname = self.keywords['name']
        mymeta = Metadata(metafile=os.path.join(myname, "metadata.txt"))
        mylabel = mymeta.search_data(label)
        if mylabel == "":
            raise MASTError(self.__class__.__name__, 
                "No metadata for tag %s" % label)
        return mylabel[1]

    def change_my_status(self, newstatus):
        """Change an ingredient status by writing the new status to 
            change_status.txt in the ingredient folder, to get picked
            up by the recipe plan.
            Args:
                newstatus <str>: New status to which to change the ingredient.
        """
        ingdir = self.keywords['name']
        oneup = os.path.dirname(ingdir)
        tryrecipe = os.path.basename(oneup)
        statuspath = ""
        if dirutil.dir_is_in_scratch(tryrecipe):
            statuspath = "%s/change_status.txt" % ingdir
        else:
            twoup = os.path.dirname(oneup)
            tryrecipe = os.path.basename(twoup)
            if dirutil.dir_is_in_scratch(tryrecipe):
                statuspath = "%s/change_status.txt" % oneup
            else:
                raise MASTError(self.__class__.__name__, "Cannot change status of ingredient %s as recipe %s or %s is not found in $MAST_SCRATCH." % (self.keywords['name'],oneup, twoup))
        if os.path.isfile(statuspath):
            statusfile = MASTFile(statuspath)
        else:
            statusfile=MASTFile()
        statusfile.data.append("%s:recommend:%s" % (newstatus, time.asctime()))
        statusfile.to_file(statuspath)
        self.logger.info("Recommending status change to %s" % newstatus)



import random
import numpy
from MAST.structopt.tools.find_defects import find_defects

def move_la(indiv, Optimizer):
    """Move function to move atoms in structure by lattice constant.  Intended for use in Defect optimization.
    Inputs:
        indiv = Individual class object to be altered
        Optimizer = Optimizer class object with needed parameters
    Outputs:
        indiv = Altered Individual class object
    """
    if 'MU' in Optimizer.debug:
        debug = True
    else:
        debug = False
    #Move structure by lattice constant
    Optimizer.output.write('Lattice Constant Move Mutation performed on individual\n')
    if Optimizer.structure=='Defect':
        if Optimizer.isolate_mutation:
            indc,indb,vacant,swaps,stro = find_defects(indiv[0],Optimizer.solidbulk,0)
            ind = indc.copy()
        else:
            ind = indiv[0]
        la = numpy.maximum.reduce(numpy.maximum.reduce(ind.get_cell())/Optimizer.solidcell)
        ax = [[1,0,0],[0,1,0],[0,0,1],[-1,0,0],[0,-1,0],[0,0,-1]]
        selax = [la*i for i in random.choice(ax)]
        ind.translate(selax)
        indiv[0]=ind.copy()
        if Optimizer.isolate_mutation:
            indiv[0].extend(indb)
    else:
        Optimizer.output.write('WARNING: Move Mutation performed on non-Defect structure\n')
    Optimizer.output.write('Index = '+repr(indiv.index)+'\n')
    Optimizer.output.write(repr(indiv[0])+'\n')
    muttype='LM'
    if indiv.energy==0:
        indiv.history_index=indiv.history_index+'m'+muttype
    else:
        indiv.history_index=repr(indiv.index)+'m'+muttype
    return indiv
    


import copy
from ase import Atom, Atoms
from MAST.structopt.inp_out.write_individual import write_individual

class Individual(object):
    """Defines class object individual for use in evolution"""
    def __init__(self, data, fitness=0, index=0, history_index='0', energy=0, tenergymx=0, 
                tenergymin=0, bulki=Atoms(), bulko=Atoms(), box=Atoms(), pressure=0, volume=0, 
                force=0, purebulkenpa=0, natomsbulk=0, fingerprint=0, swaplist=[],
                vacancies=Atoms(),swaps=Atoms()):
        self.fitness=fitness
        self.index=index
        self.history_index=history_index
        self.energy=energy
        self.tenergymx=tenergymx
        self.tenergymin=tenergymin
        self.bulki=bulki
        self.bulko=bulko
        self.box=box
        self.pressure=pressure
        self.volume=volume
        self.force=force
        self.purebulkenpa=purebulkenpa
        self.natomsbulk=natomsbulk
        self.fingerprint=fingerprint
        self.swaplist=swaplist
        self.vacancies=vacancies
        self.swaps=swaps
        self.data=[data]
    
    def __getitem__(self, i):
        return self.data[i]
    
    def __setitem__(self, key, item):
        self.data[key]=item
    
    def __iter__(self):
        return self
    
    def __copy__(self):
        self.data=copy.copy(self.data)
        return self.data
    
    def __deepcopy__(self):
        self.data=copy.deepcopy(self.data)
        return self.data
    
    def __setattr__( self, attr, value ):
        super( Individual, self ).__setattr__( attr, value )
    
    def next(self):
        if self.index > len(self.data):
            raise StopIteration
        self.index=self.index+1
        return self.data[self.index]
    
    def len(self):
        return len(self.data)
    
    def duplicate(offspring):
        """Duplicates individual"""
        dupli=offspring[0].copy()
        dup=Individual(dupli)
        dup.fitness=copy.copy(offspring.fitness)
        dup.index=copy.copy(offspring.index)
        dup.history_index=copy.copy(offspring.history_index)
        dup.energy=copy.copy(offspring.energy)
        dup.tenergymx=copy.copy(offspring.tenergymx)
        dup.tenergymin=copy.copy(offspring.tenergymin)
        dup.bulki=offspring.bulki.copy()
        dup.bulko=offspring.bulko.copy()
        dup.box=offspring.box.copy()
        dup.pressure=copy.copy(offspring.pressure)
        dup.volume=copy.copy(offspring.volume)
        dup.force=copy.copy(offspring.force)
        dup.purebulkenpa=copy.copy(offspring.purebulkenpa)
        dup.natomsbulk=copy.copy(offspring.natomsbulk)
        dup.fingerprint=copy.copy(offspring.fingerprint)
        dup.swaplist=copy.deepcopy(offspring.swaplist)
        dup.vacancies=offspring.vacancies.copy()
        dup.swaps=offspring.swaps.copy()
        return dup
    
#     def write(self,indivfile):
#         write_individual(self,indivfile)
#         return
    
#     def read(self,indivfile):
#         self = read_individual(indivfile)
#         return self





import numpy
import os
import sys
import copy
import random
from tempfile import mkdtemp
import math
import cmath
import matplotlib
matplotlib.use('Agg')
import matplotlib.pyplot as plt
import matplotlib.cm  as cm
from PIL import Image, ImageChops
from ase.io import write
from ase import Atom, Atoms
import scipy.interpolate
import scipy.special
import scipy.misc
import scipy.ndimage as ndimage
import scipy.ndimage.filters as filters
from scipy import optimize

class ConvStem():
    def __init__(self, parameters={}, mode='Simple', tmp_dir=None, keep_files=True, 
        calc_exp=True, debug=False):
        """Inputs:
        parameters - dictionary of parameters for ConvStem
        temp-dir - directory string to store ConvStem Files
        keep-Files - True/False for whether or not to keep files after run"""
        #Initialize parameters
        self.parameters=parameters
        self.keep_files=keep_files
        self.mode=mode
        self.debug=debug
        #Initialize directory for storing data
        if tmp_dir is None:
            self.tmp_dir = mkdtemp(prefix='ConvStem-')
        else:
            self.tmp_dir=os.path.realpath(tmp_dir)
            if not os.path.isdir(self.tmp_dir):
                os.mkdir(self.tmp_dir, 0755)
        #Get the probe function based on input parameters
        self.psf = get_probe_function(self.parameters)
        if calc_exp:
            if self.mode=='Simple':
                self.expfun=self.calculate_simp_function(self.parameters['Exp_Image'])
            elif self.mode=='Complex':
                self.expfun=self.calculate_comp_function(self.parameters['Exp_Image'])
            else:
                raise RuntimeError('Error: must specify either Simple or Complex mode')
            if self.keep_files:
                fig = plt.figure()
                ax = fig.add_subplot(111)
                ax.imshow(self.expfun,cmap=cm.hot)
                ax.set_frame_on(False)
                ax.set_xticks([])
                ax.set_yticks([])
                plt.axis('off')
                ax.set_aspect('equal')
                figname = 'Experimental_Image.png'
                fig.savefig(self.tmp_dir+'/'+figname, transparent=True, bbox_inches='tight',pad_inches=0)
            
    def calculate_simp_function(self,image):
        #Load image file and convert image to grayscale
        #ima=Image.open(image).convert('LA')
        #a=numpy.array(ima)
        #a=scipy.misc.imread(image)
        #Get data array of image values
        #data=numpy.zeros((len(a),len(a[0])))
        #for i in range(len(a)):
        #    for j in range(len(a[0])):
        #        data[i][j]=a[i][j][0]
        #return data
        return image
    
    def calculate_comp_function(self,imagefile):
        #Function to calculate the overall gaussian function of an ConvStem simulation
        
        #Load image file and convert image to grayscale
        img=Image.open(imagefile).convert('LA')
        
        #Add ImageChops to account for strange pbc issue
        #imoff=ImageChops.offset(img,img.size[0]/2,img.size[1]/2)
        #a=numpy.array(imoff)
        
        #Get data array of image values
        data=numpy.zeros((len(a),len(a[0])))
        for i in range(len(a)):
            for j in range(len(a[0])):
                data[i][j]=a[i][j]
        
        #Calculate the atom column positions
        positions=get_atom_pos(self,data)
        x=positions[0]
        y=positions[1]
        
        #Apply rotation/translation
        mvs=[]
        for i in range(len(x)):
            mvs.append(data[y[i],x[i]])
        top2=sorted(enumerate(mvs), key=lambda a:a[1], reverse=True)[:2]
        #Set new origin at max value
        #Translation vector becomes:
        trans=[x[top2[0][0]],y[top2[0][0]]]
        #Find rotation angle alpha
        slope=float(y[top2[0][0]]-y[top2[1][0]])/float(x[top2[0][0]]-x[top2[1][0]])
        alpha=math.atan(abs(slope))
        #Adjust for quadrant
        if x[top2[1][0]] < x[top2[0][0]]:
            if y[top2[1][0]] > y[top2[0][0]]:
                #quadrant 2
                alpha=alpha+math.pi/2.0
            else:
                #quadrant 3
                alpha=alpha+math.pi
        else:
            if y[top2[1][0]] < y[top2[0][0]]:
                #quadrant 4
                alpha=alpha+math.pi*3.0/2.0
        xs=numpy.arange(len(data))
        xys=[]
        z=[]
        for i in range(len(xs)):
            for j in range(len(xs)):
                ara=numpy.array([xs[i], xs[j], 1])
                xys.append(ara)
                z.append(data[j][i])
        transfmat=numpy.array([[math.cos(alpha), -1.0*math.sin(alpha), -1.0*x[top2[0][0]]],[math.sin(alpha),math.cos(alpha),-1.0*y[top2[0][0]]],[0.0,0.0,1.0]])
        nx=[]
        ny=[]
        for one in xys:
            ara=numpy.dot(transfmat,one)
            nx.append(ara[0])
            ny.append(ara[1])
        
        #Calculate a gaussian intensity function for each atom
        #gauss=[]
        #for one in positions:
        #    gfun=get_gaussian(one)
        #    gauss.append(gfun)
        #Merge gaussian functions
        #gfuntot=merge_gauss(gauss)
        
        gfuntot=scipy.interpolate.RectBivariateSpline(x,y,z)
        
        return gfuntot
    
    def compare_functions(self, expfun, simfun):
        """Function compares two matrices and calculates chisq.  Matrices must be same size"""
        chi=[]
        for i in range(len(expfun)):
            for j in range(len(expfun[0])):
                c=(simfun[i][j]-expfun[i][j])**2/expfun[i][j]
                chi.append(c)
        chisq=sum(chi)
        return chisq
    
    def run(self,atms):
        #Get the image from ConvStem
        simfun=self.get_image(self.psf,atms,self.parameters['Slice size'],self.parameters['Pixels'])
        #Calculate the function for the image
        #if self.mode=='Simple':
        #    simfun=self.calculate_simp_function(image)
        #elif self.mode=='Complex':
        #    simfun=self.calculate_comp_function(image)
        chisq=self.compare_functions(self.expfun,simfun)
        if self.keep_files:
            fig = plt.figure()
            ax = fig.add_subplot(111)
            #Uncomment the following lines to save image as borderless png
            ax.imshow(simfun,cmap=cm.hot)
            ax.set_frame_on(False)
            ax.set_xticks([])
            ax.set_yticks([])
            plt.axis('off')
            ax.set_aspect('equal')
            fignamep = 'SimImage-'
            #Label image with next available number
            x=0
            while True:
                figname = fignamep+repr(x)+'.png'
                if not os.path.exists(self.tmp_dir+'/'+figname):
                    break
                else:
                    x+=1
            fig.savefig(self.tmp_dir+'/'+figname, transparent=True, bbox_inches='tight',pad_inches=0)
        return chisq
    
    def get_image(self, psf, atms, rmax, nx, scalefactor=1.0):
        """Function to get image based point spread function and atoms
        rmax=Size of slice in Angstoms
        nx=number of pixels in slice"""
        #dr =  rmax / nx
        #ry = numpy.arange(0.0,rmax,dr)
        pot = stempot(rmax,rmax,len(psf),len(psf[0]),atms,scalefactor)
        #pot = stempot(xyz, rmax, rmax, nk, nk)
        potft = numpy.fft.fft2(pot)
        psf2d = numpy.fft.fft2(psf)
        potm = potft*psf2d
        zcon_im = numpy.fft.ifft2(potm)
        zcon_r = numpy.real(zcon_im)
        #zcon_s = numpy.fft.fftshift(zcon_r)
        zcon_s = zcon_r
        #grid = zcon_s.reshape((len(ry), len(ry)))
        #SetScale/P x DimOffSet(zcon_im, 0), DimDelta(zcon_im, 0), "", zcon_s
        #SetScale/P y DimOffSet(zcon_im, 1), DimDelta(zcon_im, 1), "", zcon_s
    #     fig = plt.figure()
    #     ax = fig.add_subplot(111)
    #     #Uncomment the following lines to save image as borderless png
    #     ax.imshow(grid,cmap=cm.hot)
    #     ax.set_frame_on(False)
    #     ax.set_xticks([])
    #     ax.set_yticks([])
    #     plt.axis('off')
    #     ax.set_aspect('equal')
    #     fignamep = 'SimImage-'
    #     #Label image with next available number
    #     x=0
    #     while True:
    #         figname = fignamep+repr(x)+'.png'
    #         if not os.path.exists(self.tmp_dir+'/'+figname):
    #             break
    #         else:
    #             x+=1
    #     fig.savefig(self.tmp_dir+'/'+figname, transparent=True, bbox_inches='tight',pad_inches=0)
        return zcon_s
    
    def get_atom_pos(self, data):
		"""Function to identify the location of the atom columns
		Inputs are:
			data = 2D matrix with imagefile value
			self.parameters:
				neighborhood_size = size of square stamp to search array
				threshold = difference for change"""
		
		# #Get data array of image values
		# data=list(ima.getdata())
		# imdat=[value for (value,max) in data]
		# width, height = ima.size
		# holder=numpy.zeros((height,width))
		# for h in height:
			# for w in width:
				# holder[h][w]=data[w+h*width]
		
		#Set values for peak guides
		if 'neighborhood_size' in self.parameters:
			neighborhood_size = self.parameters['neighborhood_size']
		else:
			neighborhood_size = 30
		if 'threshold' in self.parameters:
			threshold = self.parameters['threshold']
		else:
			threshold = 30
		
		#Use filters to calculate peaks
		data_max = filters.maximum_filter(data, neighborhood_size)
		maxima = (data == data_max)
		data_min = filters.minimum_filter(data, neighborhood_size)
		diff = ((data_max - data_min) > threshold)
		maxima[diff == 0] = 0

		labeled, num_objects = ndimage.label(maxima)
		slices = ndimage.find_objects(labeled)
		x, y = [], []
		for dy,dx in slices:
			x_center = (dx.start + dx.stop - 1)/2
			x.append(x_center)
			y_center = (dy.start + dy.stop - 1)/2    
			y.append(y_center)
		
		#Plot peaks points overlayed on image
		#plt.imshow(data)
		#plt.scatter(x,y)
		#plt.show()
		
		#positions=[[x[i],y[i]] for i in range(len(x))]
		posiitons=[x,y]
		
		return positions
    
def get_probe_function(parameters):
    """Function to get the probe function based on input parameters
    kev=Electron energy in keV
    ap = Objective aperture semiangle in mrad
    Cc = Chromatic aberration coefficient in mm
    dE = Delta E in eV
    ds=Source size in Angstroms
    rmax=Size of slice in Angstroms
    Cs=Spherical aberration Cs in mm
    df=Defocus in Angstroms
    nx=number of pixels in slice"""
    
    keV = parameters['Electron energy']
    ap = parameters['Aperture semiangle']
    try:
        Cc = parameters['Chromatic aberration Coefficient']
    except:
        Cc = 0.0
    try:
        dE = parameters['Delta E']
    except:
        dE = 0.0
    #Cs=parameters['Spherical aberration']
    #df=parameters['Defocus']
    ds=parameters['Source size']
    rmax=parameters['Slice size']
    #nx=parameters['Pixels']
    try:
        aber=parameters['aber']
    except:
        aber=[[0,0] for i in range(12)]
    
    nk = math.floor(40*(0.001*ap / wavlen(keV))*rmax)
    if math.fmod(nk, 2):
        nk += 1
    
    #Get probe function
    if Cc == 0.0 and dE == 0.0:
        psf2D = STEMPSF2DCoh(aber, keV, ap, nk)
        #Redimension/D/C probe2DCoh #Converts wave to double precision complex wave
        #wave/c psf2D = $"probe2DCoh"
    else:
        psf2D = STEMPSF2DIncoh(aber, keV, Cc, dE, ds, ap, nk)
        #Redimension/D/C probe2DIncoh #Converts wave to double precision complex wave
        #wave/c psf2D = $"probe2DIncoh"
    #Shift minimum to zero
    mn = numpy.minimum.reduce(numpy.minimum.reduce(psf2D))
    for i in range(len(psf2D)):
        for j in range(len(psf2D[0])):
            psf2D[i][j] += abs(mn)
    
    return psf2D

def STEMPSF2DCoh(aber, keV, ap, nk):
    kmax = 0.001*ap / wavlen(keV)
    phase = ChiPhase2D(aber, keV, ap, nk/10)
    xp=numpy.linspace(-kmax*2.0,(kmax*2.0),nk/10)
    yp=numpy.linspace(-kmax*2.0,(kmax*2.0),nk/10)
    phasef = scipy.interpolate.RectBivariateSpline(xp,yp,phase)
    
    # need to pad the phase with zeros here.
    probe2DCoh = numpy.zeros((nk,nk),dtype=complex)
    x=numpy.linspace(-20*kmax,40*kmax,nk)
    y=numpy.linspace(-20*kmax,40*kmax,nk)
    
    #SetScale/I x -20*kmax, 40*kmax, "", probe2DCoh
    #SetScale/I y -20*kmax, 40*kmax, "", probe2DCoh
    for i in range(len(x)):
        for j in range(len(y)):
            if x[i]**2 + y[j]**2 < kmax**2:
                probe2DCoh[i][j] = complex(1, phasef(x[i],y[j]))
            else:
                probe2DCoh[i][j] = complex(0, 0)
    #probe2DCoh = ( (x^2 + y^2 < kmax^2) ?  p2rect(cmplx(1, phase(x)(y)))  : cmplx(0, 0))

    p2dcf = numpy.fft.fft2(probe2DCoh)
    for i in range(len(x)):
        for j in range(len(y)):
            probe2DCoh[i][j] = complex((abs(p2dcf[i][j]))**2, 0)
    probe2DCoh = numpy.real(probe2DCoh)    
    probe2DCoh /= sum(sum(probe2DCoh))
    
    return probe2DCoh

def STEMPSF2DIncoh(aber, keV, Cc, dE, ds, ap, nk):
    kmax = 0.001*ap / wavlen(keV)
    defocus_distribution = ChromaticDefocusDistribution(Cc, dE, keV, ap)
    start_df = aber[0][0]
    df_range = 2.5*(Cc*dE /(1e3*keV)*( (1+keV/511)/(1+keV/1022) ))
    x = numpy.linspace(-df_range,df_range, num=len(defocus_distribution))
    for i in range(len(defocus_distribution)):
        aber[0][0] = start_df + x[i]
        probe2DCoh = STEMPSF2DCoh(aber, keV, ap, nk)
        if i==0:
            probe2DIncoh = copy.deepcopy(probe2DCoh)
            probe2DIncoh *= defocus_distribution[0]
        else:
            for j in range(len(probe2DIncoh)):
                for k in range(len(probe2DIncoh[0])):
                    probe2DIncoh[j][k] += defocus_distribution[i]*probe2DCoh[j][k]
    
    probe2DIncoh /= sum(defocus_distribution)

    aber[0][0] = start_df

    if ds != 0:
        fds = ds / (2*(2*math.log(2))**0.5)    # real-space standard deviation for Gaussian with FWHM ds
        fds = 1/(2*math.pi*fds)    # FT standard deviation
        #Redimension/C probe2DIncoh
        #wave/C probe2DSS = $"probe2DIncoh"
        probe2DSS = numpy.fft.fft2(probe2DIncoh)
        g = Gauss(0.0, 0.0, fds, fds)#, len(probe2DSS), len(probe2DSS[0]))
        xs = numpy.linspace(-1.0, 1.0, num=len(probe2DSS))
        for j in range(len(probe2DSS)):
                for k in range(len(probe2DSS[0])):
                    probe2DSS[j][k]*=complex(g(xs[j],xs[k]),0.0)
        #probe2DSS *= complex(Gauss(x, 0.0, fds, y, 0.0, fds), 0.0)
        probe2DSS = numpy.fft.ifft2(probe2DSS)
        probe2DSS = numpy.real(probe2DSS)
        probe2DSS = numpy.fft.fftshift(probe2DSS)
        #probe2DSS = cmplx(sqrt(magsqr(probe2DSS)), 0)
        probe2DSS /= sum(sum(probe2DSS))
        probe2DIncoh = copy.deepcopy(probe2DSS)
        #SetScale/p x dimoffset(probe2dcoh, 0), dimdelta(probe2dcoh, 0), "", probe2dIncoh
        #SetScale/P y dimoffset(probe2dcoh, 1), dimdelta(probe2dcoh, 1), "", probe2dIncoh
    
    return probe2DIncoh

def ChiPhase2D(aber, keV, ap, nk):
    aber = SwitchToAngstroms(aber)
    wl = wavlen(keV)
    kmax = (0.001*ap / wl)  # maximum k through aperture
    x=numpy.linspace(-kmax*2.0,(kmax*2.0),nk)
    y=numpy.linspace(-kmax*2.0,(kmax*2.0),nk)
    astack = [numpy.zeros((len(x),len(y))) for one in range(12)]
    
    # Evaluate the phase shifts from the various aberrations
    for i in range(len(x)):
        for j in range(len(y)):
            #C1, defocus
            astack[0][i][j] = (1.0/2.0)*wl*aber[0][0]*(x[i]**2.0 + y[j]**2.0)
            #A1, 2-fold astigmatism
            astack[1][i][j] = (1.0/2.0)*wl*aber[1][0]*(x[i]**2.0 - y[j]**2.0)
            #A2, 3-fold astigmatism
            astack[2][i][j] = (1.0/3.0)*wl**2.0*aber[2][0]*(x[i]**3.0 - 3.0*x[i]*y[j]**2.0)
            #B2, axial coma
            astack[3][i][j] = wl**2.0*aber[3][0]*(x[i]**3.0 + x[i]*y[j]**2.0)
            #C3, primary spherical aberration
            astack[4][i][j] = (1.0/4.0)*wl**3.0*aber[4][0]*(x[i]**4.0 + 2.0*x[i]**2.0*y[j]**2.0 + y[j]**4.0)
            #A3, 4-fold astigmatism
            astack[5][i][j] = (1.0/4.0)*wl**3.0*aber[5][0]*(x[i]**4.0 - 6.0*x[i]**2.0*y[j]**2.0 + y[j]**4.0)
            #S3, star aberration
            astack[6][i][j] = wl**3.0*aber[6][0]*(x[i]**4.0 - y[j]**4.0)
            #A4, 5-fold astigmatism
            astack[7][i][j] = (1.0/5.0)*wl**4.0*aber[7][0]*(x[i]**5.0 - 10.0*x[i]**3.0*y[j]**2.0 + 5.0*x[i]*y[j]**4.0)
            #D4, 3-lobe aberration
            astack[8][i][j] = wl**4.0*aber[8][0]*(x[i]**5.0 - 2.0*x[i]**3.0*y[j]**2.0 - 3.0*x[i]*y[j]**4.0)
            #B4, axial coma
            astack[9][i][j] = wl**4.0*aber[9][0]*(x[i]**5.0 + 2.0*x[i]**3.0*y[j]**2.0 + x[i]*y[j]**4.0)
            #C5, 5th order spherical aberration
            astack[10][i][j] = (1.0/6.0)*wl**5.0*aber[10][0]*(x[i]**6.0 + 3.0*x[i]**4.0*y[j]**2.0 + 3.0*x[i]**2.0*y[j]**4.0 + y[j]**6.0)
            #A5, 5th order spherical aberration
            astack[11][i][j] = (1.0/6.0)*wl**5.0*aber[11][0]*(x[i]**6.0 - 15.0*x[i]**4.0*y[j]**2.0 + 15.0*x[i]**2.0*y[j]**4.0 - y[j]**6.0)
    
    #Set minimum to zero
    #nnastack = [numpy.zeros((len(x),len(y))) for one in range(12)] 
    # for i in range(len(x)):
#         for j in range(len(y)):
#             for k in range(len(astack)):
#                 if astack[k][i][j] < 0.0:
#                     astack[k][i][j]=0.0
    #Shift minimum to zero
    for k in range(len(astack)):
        mn = numpy.minimum.reduce(numpy.minimum.reduce(astack[k]))
        for i in range(len(x)):
            for j in range(len(y)):
                astack[k][i][j]+= abs(mn)
    
    # rotate the phase shifts of the non-centrosymmetric aberrations
    for i in range(12):
        if aber[i][1] != 0.0:
            astack[i] = scipy.ndimage.rotate(astack[i],-aber[i][1])
            #ImageTransform/P=(i) getplane astack
            #wave aphase = $"M_ImagePlane"
            #ImageRotate/E=0/O/A=(aber[i][1]) aphase
            #SetScale/P x -DimSize(aphase, 0)*DimDelta(astack, 0) / 2, DimDelta(astack, 0), "", aphase
            #SetScale/P y -DimSize(aphase, 1)*DimDelta(astack, 1) / 2, DimDelta(astack, 0), "", aphase
            #astack[][][i] = aphase(x)(y)

    # sum all the aberration contributions
    fsum = numpy.zeros((len(x),len(y)))
    for i in range(len(x)):
        for j in range(len(y)):
            fsum[i][j] = sum([one[i][j] for one in astack])*2*math.pi
    #MatrixOp/O phase = 2*Pi*sumbeams(astack)
    #SetScale/I x -2*kmax, 2*kmax, "", phase
    #SetScale/I y -2*kmax, 2*kmax, "", phase
    
    return fsum

def ChromaticDefocusDistribution(Cc, dE, keV, ap):
    Cc *= 1e7  # mm to Angstroms
    df_phase_max = 2*math.pi / 50  # maximum phase step at the aperture edge due to chromatic aberration
    kmax = 0.001*ap/wavlen(keV)
    # defocus range and form from Reimer
    H = (Cc*dE /(1e3*keV)*( (1+keV/511)/(1+keV/1022) ))
    N = (2*(math.log(2)**0.5) / ((math.pi**0.5)*H))
    df_range = 2.5*H
    ndf = math.ceil(df_range * wavlen(keV) * kmax**2 / df_phase_max)
    if ndf < 31:
        ndf = 31
    
    #ndf = (ndf < 31 ? 31 : ndf)
    if not math.fmod(ndf,2):
        ndf+=1
    
    #ndf = (!mod(ndf, 2) ? ndf+1 : ndf)
    defocus_distribution = numpy.linspace(-df_range, df_range, num = ndf)
    #Make/O/N=(ndf) defocus_distribution
    #SetScale/I x -df_range, df_range, "", defocus_distribution
    for x in range(len(defocus_distribution)):
        defocus_distribution[x] = N*math.exp( -math.log(2) * (2*defocus_distribution[x] / H)**2 )
    
    return defocus_distribution

def stempot(xmax,ymax,nx,ny,atms,scalefactor):
    """function V = stempot(xmax,ymax,nx,ny,potfile)
    %  STEMPOT Generate a Projected Potential
    %  inputs xmax, ymax are the size of the slice in angstroms
    %  nx,ny are the number of pixels in the x and y directions """
    #zed=2 for rutherford scattering of the nucleus, less for screening
    zed = 1.7

    ix = numpy.arange(1.0,nx)
    iy = numpy.arange(1.0,ny)
    dx = xmax/nx
    dy = ymax/ny
    rx = numpy.arange(0,xmax-dx,dx)
    ry = numpy.arange(0,ymax-dy,dy)

    Zatom = atms.get_atomic_numbers()
    #translate atoms such that the center of mass is in the center of the computational cell
    com = atms.get_center_of_mass()
    cop = xmax/2.0
    trans = [cop-i for i in com]
    atms.translate(trans)
    positions=atms.get_positions()
    ax=[]
    ay=[]
    az=[]
    for o,t,h in positions:
        ax.append(o)
        ay.append(t)
        az.append(h)
    ax = numpy.array(ax)
    ay = numpy.array(ay)
    az = numpy.array(az)
    amax = len(Zatom)

    #find boundaries of slice
    axmin = min(ax)
    axmax = max(ax)
    aymin = min(ay)
    aymax = max(ay)
    
    V= numpy.zeros((nx,ny))

    #map x and y coords of the atoms to the nearest grid points
    #A fraction of the atom must be assigned to the closest gridpoints
    #to avoid sum and difference frequencies appearing in the image
    #grid point to the left of the atom
    iax = numpy.array([math.floor(axi/dx)+1 for axi in ax])
    #create periodic boundary conditions
    ibx=numpy.array([math.fmod(iaxi,nx)+1 for iaxi in iax])
    #fraction of atom at iax
    fax = numpy.array([1-math.fmod((axi/dx),1 ) for axi in ax]) 
    #grid point above the atom
    iay = numpy.array([math.floor(ayi/dy)+1 for ayi in ay])
    #create periodic boundary conditions
    iby = numpy.array([math.fmod(iayi,ny)+1 for iayi in iay])
    #fraction of atom at iay 
    fay = numpy.array([1-math.fmod((ayi/dy),1 ) for ayi in ay])
    
    #Add each atom to the potential grid
    V1 = numpy.array([fax[i] * fay[i] * (Zatom[i]**zed) for i in range(len(fax))])
    V2 = numpy.array([(1-fax[i]) * fay[i] * (Zatom[i]**zed) for i in range(len(fax))])
    V3 = numpy.array([fax[i] * (1-fay[i]) * (Zatom[i]**zed) for i in range(len(fax))])
    V4 = numpy.array([(1-fax[i]) * (1-fay[i]) * (Zatom[i]**zed) for i in range(len(fax))])
#     V1 = numpy.array([fax[i] * fay[i] * scalefactor for i in range(len(fax))])
#     V2 = numpy.array([(1-fax[i]) * fay[i] * scalefactor for i in range(len(fax))])
#     V3 = numpy.array([fax[i] * (1-fay[i]) * scalefactor for i in range(len(fax))])
#     V4 = numpy.array([(1-fax[i]) * (1-fay[i]) * scalefactor for i in range(len(fax))])

    for j in range(amax):
       V[iax[j],iay[j]] += V1[j]
       V[ibx[j],iay[j]] += V2[j]
       V[iax[j],iby[j]] += V3[j]
       V[ibx[j],iby[j]] += V4[j]
    
    return V

def wavlen(keV):
    #calculate electron wavelength in Angstroms given the energy in keV.
    wav = 12.3986 / ( (2*511.0 + keV) * keV)**0.5
    return wav

def SwitchToAngstroms(aber):
    '''Function to switch aberrations wave into Angstroms instead of natural units'''
    naber = [[0.0,0.0] for i in range(12)]
    #C1, A1, A3, B2, all start in nm
    for i in range(0,4):
        naber[i][0] = 10*aber[i][0]
    #C3, A3, S3, A4, D4, B4 start in um
    for i in range(4,10):
        naber[i][0] = 1e4*aber[i][0]
    #C5, A5 in mm
    for i in range(10,12):
        naber[i][0] = 1e7*aber[i][0]
    
    return naber

def Gauss(center_x, center_y, width_x, width_y, height=1.0):
    """Returns a gaussian function with the given parameters"""
    width_x = float(width_x)
    width_y = float(width_y)
    return lambda x,y: height*math.exp(-(((center_x-x)/width_x)**2+((center_y-y)/width_y)**2)/2)

def moments(data):
	"""Returns (height, x, y, width_x, width_y)
		the gaussian parameters of a 2D distribution by calculating its moments """
	total = data.sum()
	X, Y = indices(data.shape)
	x = (X*data).sum()/total
	y = (Y*data).sum()/total
	col = data[:, int(y)]
	width_x = sqrt(abs((arange(col.size)-y)**2*col).sum()/col.sum())
	row = data[int(x), :]
	width_y = sqrt(abs((arange(row.size)-x)**2*row).sum()/row.sum())
	height = data.max()
	return x, y, width_x, width_y, height

def fitgaussian(data):
	"""Returns (height, x, y, width_x, width_y)
		the gaussian parameters of a 2D distribution found by a fit"""
	params = moments(data)
	errorfunction = lambda p: ravel(Gauss(*p)(*indices(data.shape)) - data)
	p, success = optimize.leastsq(errorfunction, params)
	return p

def find_stem_coeff(Optimizer, indiv):
    from MAST.structopt.tools.eval_energy import eval_energy
    outs = eval_energy([Optimizer,indiv])
    indiv.energy = outs[0]
    stro=outs[3]
    if Optimizer.structure == 'Defect' or Optimizer.structure=='Surface':
        indiv.bulki = outs[1]
    chisq = Optimizer.stemcalc.run(indiv[0])
    aflag=True
    alpha = 1.0
    while True:
        value=alpha*chisq
        div=abs(indiv.energy)/value
        if div <1:
            alpha*=0.1
        elif div >10:
            alpha*=10
        else:
            break
    indiv.fitness=indiv.energy+alpha*chisq
    return alpha, indiv



'''Functions for use in optimizer'''
#import crossover
#import fingerprinting
#import fitness
#import generate
#import inp_out
#import moves
#import predator
#import selection
#import switches
#import tools
#from Optimizer import Optimizer



##############################################################
# This code is part of the MAterials Simulation Toolkit (MAST)
# 
# Maintainer: Glen Jenness
# Last updated: 2013-07-01
##############################################################
from MAST.utility.defect_formation_energy.defectformationenergy import DefectFormationEnergy
from MAST.utility.defect_formation_energy.potential_alignment import PotentialAlignment
from MAST.utility.defect_formation_energy.gapplot import GapPlot



import numpy
from ase import Atoms, Atom

def CCP_Oct_Tet(ind1, ind2, Optimizer):
    """CX Function underdevelopment for preferentially exchanging octahedral and tetrahedral defect sites
    Non-Functional
    """
    if 'CX' in Optimizer.debug:
        debug = True
    else:
        debug = False
    uc = numpy.maximum.reduce(Optimizer.solidbulk.get_cell()/Optimizer.supercell)[0]
    interstitials = [[0.0,0.5,0.0],[0.5,0.0,0.0],[0.0,0.0,0.5],[0.5,0.5,0.5],
                    [0.25,0.25,0.25],[0.25,0.75,0.25],[0.75,0.25,0.25],[0.75,0.75,0.25],
                    [0.25,0.25,0.75],[0.25,0.75,0.75],[0.75,0.25,0.75],[0.75,0.75,0.75]]
    sites=[]
    for one in interstitials:
        pos = [uc*p for p in one]
        npos = [0,0,0]
        for i in range(Optimizer.supercell[0]):
            npos[0]=pos[0]+i*uc
            for j in range(Optimizer.supercell[1]):
                npos[1]=pos[1]+j*uc
                for k in range(Optimizer.supercell[2]):
                    npos[2]=pos[2]+k*uc
                    sites.append(copy.copy(npos))
    solidsites = Atoms()
    for one in sites:
        solidsites.append(Atom(position=one))
    return ind1, ind2



##############################################################
# This code is part of the MAterials Simulation Toolkit (MAST)
# 
# Maintainer: Tam Mayeshiba
# Last updated: 2014-04-25
##############################################################
import os
import time
import logging
import inspect
import subprocess
import importlib
from MAST.ingredients.chopingredient import ChopIngredient
from MAST.summary import citations
#from MAST.ingredients.chopingredient import WriteIngredient
#from MAST.ingredients.chopingredient import IsReadyToRunIngredient
#from MAST.ingredients.chopingredient import RunIngredient
#from MAST.ingredients.chopingredient import IsCompleteIngredient
#from MAST.ingredients.chopingredient import UpdateChildrenIngredient
from MAST.utility import MASTFile
from MAST.utility import MASTError
from MAST.utility import loggerutils
class RecipePlan:
    """Contains the entire recipe plan.
        Attributes:
            self.name <str>: Recipe name
            self.ingredients <dict>: Dictionary of ingredients
                                     and their statuses
            self.update_methods <dict>: Dictionary of each
                  ingredient's update method, per child
                  [ingredient][child] = list of update
                  methods for the child
                  the method list is a nested list:
                  [[method1name,arg1,arg2,...][method2name,...]]
            self.parents_to_check <dict>: Dictionary of parents
                  to check for completion before each ingredient
                  may be run
            self.run_methods <list of list>: List of ingredient
                                     run methods
            self.write_methods <list of list>: List of ingredient
                                       write methods
            self.ready_methods <list of list>: List of ingredient
                                       ready methods
            self.complete_methods <list of list>: List of ingred.
                                          complete methods
            self.ingred_input_options <dict>: Dictionary of 
                       ingredient input options
            self.status <str>: Recipe status
            self.working_directory <str>: Recipe working directory
            self.logger <logging logger>: Control-level logger
            self.recipe_logger <logging logger>: Recipe-level logger
            self.summary_options <InputOptions>: summary input options
    """
    def __init__(self, name, working_directory):
        self.name            = name
        self.ingredients     = dict()  #name: status
        self.update_methods   = dict()
        self.parents_to_check= dict()
        self.run_methods      = dict()
        self.write_methods    = dict()
        self.ready_methods    = dict()
        self.complete_methods = dict()
        self.ingred_input_options = dict()
        self.summary_options = ""
        self.status="I"
        self.working_directory = working_directory
        self.logger = logging.getLogger('mastmon')
        self.logger = loggerutils.add_handler_for_control(self.logger)
        self.recipe_logger = logging.getLogger(self.working_directory)
        self.recipe_logger = loggerutils.add_handler_for_recipe(self.working_directory, self.recipe_logger)

    def do_ingredient_methods(self, iname, methodtype, childname=""):
        """Do the ingredient methods.
            Args:
                iname <str>: ingredient name
                methodtype <str>: method type (write, run, etc.)
            The method list should have the format:
            [[method1name,arg1,arg2,...][method2name,arg1,...]]
        """
        if methodtype == 'mast_write_method':
            mlist = self.write_methods[iname]
        elif methodtype == 'mast_run_method':
            mlist = self.run_methods[iname]
        elif methodtype == 'mast_ready_method':
            mlist = self.ready_methods[iname]
        elif methodtype == 'mast_complete_method':
            mlist = self.complete_methods[iname]
        elif methodtype == 'mast_update_children_method':
            mlist = self.update_methods[iname][childname]
        else:
            raise MASTError(self.__class__.__name__,"Bad call to do_ingredient_methods with method type %s" % methodtype)
        allresults = list()
        self.recipe_logger.info("Do methods for %s" % methodtype)
        for methoditem in mlist:
            minputs = list(methoditem[1:])
            if methodtype == 'mast_update_children_method':
                minputs.append(childname)
            mresult = self.run_a_method(iname, methoditem[0], minputs)
            allresults.append(mresult)
        return allresults

    def run_a_method(self, iname, methodstring, minputs):
        """Run a method. Evaluates the method to run.
            Args:
                iname <str>: ingredient name
                methodstring <str>: string of class.method, e.g.:
                        ChopIngredient.write_singlerun
                        If class is not given,
                        ChopIngredient is assumed.
                    Will look in MAST.ingredients.
                minputs <list>: method inputs
            Returns:
                Results of the method, whatever it returns.
        """
        self.recipe_logger.info("Attempt to run method %s with inputs %s" % (methodstring, minputs))
        len_inputs = len(minputs)
        #Is it a ChopIngredient method?
        myclass = ""
        methodname = ""
        if not '.' in methodstring:
            myclass = "ChopIngredient"
            methodname = methodstring
        else:
            myclass = methodstring.split('.')[0]
            methodname = methodstring.split('.')[1]
        choplib_mast = importlib.import_module("MAST.ingredients")
        chopmods_mast = inspect.getmembers(choplib_mast, predicate=inspect.ismodule)
        chopclasses = list()
        for chopmod_mast in chopmods_mast:
            chopclasses.extend(inspect.getmembers(chopmod_mast[1], predicate=inspect.isclass))
        for classitem in chopclasses:
            if classitem[0] == myclass:
                mymembers = inspect.getmembers(classitem[1], predicate=inspect.ismethod)
                for classmember in mymembers:
                    if methodname == classmember[0]:
                        my_ing = classitem[1](name = self.ingred_input_options[iname]['name'], program_keys = self.ingred_input_options[iname]['program_keys'], structure = self.ingred_input_options[iname]['structure'])
                        if len_inputs == 0:
                            mresult = classmember[1](my_ing)
                        elif len_inputs == 1:
                            mresult = classmember[1](my_ing, minputs[0])
                        elif len_inputs == 2:
                            mresult = classmember[1](my_ing, minputs[0], minputs[1])
                        elif len_inputs == 3:
                            mresult = classmember[1](my_ing, minputs[0], minputs[1], minputs[2])
                        elif len_inputs == 4:
                            mresult = classmember[1](my_ing, minputs[0], minputs[1], minputs[2], minputs[3])
                        else:
                            raise MASTError(self.__class__.__name__, "Function %s for ChopIngredient requires too many inputs (> 4)." % methodname)
                        self.recipe_logger.info("Results for method name %s: %s" % (methodname, mresult))
                        return mresult
        raise MASTError(self.__class__.__name__,"Could not find method %s in class %s" % (methodname, myclass))
        return None




    def write_ingredient(self, iname):
        """Write the ingredient files according to the 
            correct method
        """
        return self.do_ingredient_methods(iname, "mast_write_method")

    def complete_ingredient(self, iname):
        """Check if an ingredient is complete
        """
        cresults = self.do_ingredient_methods(iname, "mast_complete_method")
        iscompletelist = list()
        for cresult in cresults:
            if (cresult == None) or (cresult == ""):
                pass
            elif cresult == False:
                return False
            elif cresult == True:
                iscompletelist.append(1)
            else:
                pass
        iscomplete = False
        if sum(iscompletelist) == len(cresults):
            iscomplete = True
        self.recipe_logger.info("Completeness evaluation: %s (from %s)" % (iscomplete, cresults))
        return iscomplete

    def ready_ingredient(self, iname):
        """Check if an ingredient is ready
        """
        rresults = self.do_ingredient_methods(iname, "mast_ready_method")
        isreadylist = list()
        for rresult in rresults:
            if (rresult == None) or (rresult == ""):
                pass
            elif rresult == False:
                return False
            elif rresult == True:
                isreadylist.append(1)
            else:
                pass
        isready = False
        if sum(isreadylist) == len(rresults):
            isready = True
        self.recipe_logger.info("Readiness evaluation: %s (from %s)" % (isready, rresults))
        return isready


    def run_ingredient(self, iname):
        """Run ingredient
        """
        return self.do_ingredient_methods(iname, "mast_run_method")

    def update_children(self, iname):
        """Update the children of an ingredient
        """
        upd_results=list()
        for childname in self.update_methods[iname].keys():
            upd_results.append(self.do_ingredient_methods(iname, "mast_update_children_method", childname))
        return upd_results

    def fast_forward_check_complete(self):
        """Check if runs are complete."""
        for iname in self.ingredients.keys():
            if not (self.ingredients[iname] in ["C", "E", "skip"]):
                if self.complete_ingredient(iname):
                    self.ingredients[iname] = "C"
                    self.recipe_logger.info("Status of %s changed to %s" % (iname, "C"))
                    self.update_children(iname)
        return


    def check_if_have_parents(self):
        """Check if runs at "Initialized" status have parents 
            and switch them to "Wait" if so; otherwise,
            switch them to "Stage"
        """
        for iname in self.ingredients.keys():
            if self.ingredients[iname] == "I":
                ptc = list(self.parents_to_check[iname])
                plen = len(ptc)
                if plen > 0:
                    self.ingredients[iname] = "W"
                    self.recipe_logger.info("Status of %s changed to %s" % (iname, "W"))
                else:
                    self.ingredients[iname] = "S"
                    self.recipe_logger.info("Status of %s changed to %s" % (iname, "S"))


    def check_if_ready_to_proceed_are_complete(self):
        """Check if ready-to-proceed ingredients are complete
        """
        for iname in self.ingredients.keys():
            if self.ingredients[iname] == "P":
                if self.status_change_recommended(iname):
                    pass
                else:
                    if self.complete_ingredient(iname):
                        self.ingredients[iname] = "C"
                        self.update_children(iname)
        return
    
    def status_change_recommended(self, iname):
        """Check if a status change is recommended for the ingredient,
            as listed in the ingredient folder/change_status.txt.
            Args:
                iname <str>: ingredient name
            Returns:
                True if a status change was recommended, and 
                    changes the status of the ingredient in self.ingredients.
                False otherwise
        """
        statuspath = os.path.join(self.working_directory, iname, "change_status.txt")
        if not os.path.isfile(statuspath):
            return False
        statusfile = MASTFile(statuspath)
        newdata=list()
        changed=False
        for sline in statusfile.data: #status:recommend:timestamp
            if not "status_changed" in sline:
                newstatus = sline.split(":")[0]
                self.ingredients[iname]=newstatus
                newline = sline + ":status_changed:" + time.asctime() + "\n"
                self.recipe_logger.info("Status of %s changed to %s" % (iname, newstatus))
                changed=True
                newdata.append(newline)
            else:
                newdata.append(sline)
        statusfile.data=list(newdata)
        statusfile.to_file(statuspath)
        return changed

    def check_if_parents_are_complete(self):
        """Check if parents of waiting ingredients are
            complete.
        """
        for iname in self.ingredients.keys():
            if self.ingredients[iname] == "W":
                okay=0
                ptc = list(self.parents_to_check[iname])
                plen = len(ptc)
                for parent in ptc:
                    if self.ingredients[parent] == "C":
                        self.recipe_logger.info("Status of %s changed to %s" % (parent, "C"))
                        okay = okay + 1
                if okay == plen:
                    self.ingredients[iname] = "S"
                    self.recipe_logger.info("Status of %s changed to %s" % (iname, "S"))

    def run_staged_ingredients(self):
        """Run staged ingredients.
        """
        for iname in self.ingredients.keys():
            if self.ingredients[iname] == "S":
                if self.complete_ingredient(iname):
                    self.ingredients[iname] = "C"
                    self.recipe_logger.info("Status of %s changed to %s" % (iname, "C"))
                    self.update_children(iname)
                else:
                    if not (self.ready_ingredient(iname)):
                        self.write_ingredient(iname)
                    if self.ready_ingredient(iname):
                        self.run_ingredient(iname)
                        self.ingredients[iname] = "P"
                        self.recipe_logger.info("Status of %s changed to %s" % (iname, "P"))

    def check_recipe_status(self, verbose=1):
        """Check ingredient statuses, and get recipe status
            I = Initialized
            W = Waiting on parents
            S = Staged
            P = ready to Proceed
            C = Complete
        """
        #self.fast_forward_check_complete()
        self.check_if_have_parents()
        self.check_if_ready_to_proceed_are_complete()
        self.check_if_parents_are_complete()
        self.run_staged_ingredients()
        self.print_status(verbose)
    def print_status(self, verbose=1):
        """Print status and set the recipe status.
        """

        total = len(self.ingredients.keys())
        totcomp=0
        totwait=0
        totproceed=0
        totinit=0
        totstage=0
        toterr=0
        totskip=0
        ilist = self.ingredients.keys()
        ilist.sort()
        statusfile = MASTFile()
        statusfile.data.append("#Statuses:\n")
        statusfile.data.append("#I = Initial state\n")
        statusfile.data.append("#W = Waiting on parents\n")
        statusfile.data.append("#S = Staged for run prep (parents complete)\n")
        statusfile.data.append("#P = ready to Proceed to queue; look for submission\n")
        statusfile.data.append("#C = Complete\n")
        statusfile.data.append("#E = Error\n")
        statusfile.data.append("#skip = Skip (from user)\n")


        if verbose == 1:
            namestring = "Recipe name: %s" % self.name
            self.recipe_logger.info(namestring)
            self.recipe_logger.info(time.asctime())
            self.logger.info(namestring)
            self.logger.info(time.asctime())
        for iname in ilist:
            if verbose == 1:
                ingstring = "%30s : %4s" % (iname, self.ingredients[iname])
                self.recipe_logger.info(ingstring)
                self.logger.info(ingstring)
            statusfile.data.append("%30s : %4s\n" % (iname, self.ingredients[iname]))
            if self.ingredients[iname] == "C":
                totcomp = totcomp + 1
            elif self.ingredients[iname] == "P":
                totproceed = totproceed + 1
            elif self.ingredients[iname] == "I":
                totinit = totinit + 1
            elif self.ingredients[iname] == "W":
                totwait = totwait + 1
            elif self.ingredients[iname] == "S":
                totstage = totstage + 1
            elif self.ingredients[iname] == "E":
                toterr = toterr + 1
            elif self.ingredients[iname] == "skip":
                totskip = totskip + 1
        #headerstring = "%8s %8s %8s %8s %8s %8s %8s= %8s" % ("INIT","WAITING","STAGED","PROCEED","COMPLETE", "ERROR", "USERSKIP", "TOTAL")
        headerstring = "%6s %6s %8s %15s %10s %7s %6s= %5s" % ("(I)nit","(W)ait","(S)taged","(P)roceed2queue","(C)omplete", "(E)rror", "(skip)", "TOTAL")
        self.recipe_logger.info(headerstring)
        self.logger.info(headerstring)
        #valuestring = "%8i %8i %8i %8i %8i %8i %8i= %8i" % (totinit, totwait, totstage, totproceed, totcomp, toterr, totskip, total)
        valuestring = "%6i %6i %8i %15i %10i %7i %6i= %5i" % (totinit, totwait, totstage, totproceed, totcomp, toterr, totskip, total)
        self.recipe_logger.info(valuestring)
        self.logger.info(valuestring)
        if totcomp == total:
            self.status = "C"
            self.run_summary_section()
        else:
            self.status = "R"
        #print "Recipe status: %s" % self.status
        statusfile.to_file(os.path.join(self.working_directory,"status.txt"))
        if toterr > 0:
            self.logger.error("ATTENTION: Recipe at %s has one or more ingredient errors. Please check the MAST_ERROR file and or mast_recipe.log file, in the recipe directory. Change status 'E' in status.txt to 'W' when the error has been resolved." % self.working_directory)
    
    def run_summary_section(self):
        """Run the summary section of the input file, if it exists,
            when the recipe is complete. Called from print_status.
            Summary options are in the form:
            self.summary_options["name keyword"]= ["keyword1",  "keyword2", ... ]
            The keywords should be the names of .py files located in
            MAST.summary.
            Each .py file should have a main function that takes in the
            ingredient's full path:
            def main(ingname="")
            Each .py's main function should then return a string as follows:
            "label (units);value"
        """
        name_keys = self.summary_options.keys()
        mname_dict=dict()
        for name_key in name_keys:
            mname_dict[name_key] = self.summary_options[name_key]
    
        result_dict=dict()
        for myingred in self.ingredients:
            fullpath = os.path.join(self.working_directory, myingred)
            result_dict[myingred]=dict()
            for name_key in name_keys:
                if name_key in myingred: #name matches
                    for method_key in mname_dict[name_key]:
                        try:
                            mymod = importlib.import_module("MAST.summary.%s" % method_key)
                        except ImportError:
                            self.logger.error("Could not find method %s in MAST.summary." % method_key)
                            continue
                        result_dict[myingred][method_key] = mymod.main(fullpath)
        summary = MASTFile()
        summary.data.append("RECIPE WORKING DIRECTORY: %s\n" % self.working_directory)
        result_names = result_dict.keys()
        result_names.sort()
        for result_name in result_names:
            #titlestring="t:%20s:" % result_name
            #valuestring="v:%20s:" % result_name
            method_keys = result_dict[result_name].keys()
            method_keys.sort()
            for method_key in method_keys:
                myresult = result_dict[result_name][method_key]
                title = myresult.split(";",1)[0].strip()
                value = myresult.split(";",1)[1].strip()
                result_line="%20s :: %20s :: %20s\n" % (result_name, title, value)
                summary.data.append(result_line)
                #titlestring = titlestring + "%20s:" % title
                #valuestring = valuestring + "%20s:" % value
            #titlestring = titlestring + '\n'
            #valuestring = valuestring + '\n'
            #summary.data.append(titlestring)
            #summary.data.append(valuestring)
        if summary.data == []:
            pass
        else:
            summary.to_file(os.path.join(self.working_directory,"SUMMARY.txt"))
        citationfile = MASTFile()
        citation_lines = citations.get_citations(self.working_directory)
        for cite_line in citation_lines:
            citationfile.data.append(cite_line)
        citationfile.to_file(os.path.join(self.working_directory,"CITATIONS.bib"))
        return

                            




    def add_ingredient(self, ingredient_name, ingredient):
        """Used to add an ingredient_object corresponding to an ingredient name
        """
        raise MASTError(self.__class__.__name__, "This function is obsolete.")
        self.ingredients[ingredient_name] = ingredient

    def get_ingredient(self, ingredient_name):
        """Used to get an ingredient_object corresponding to an ingredient name
        """
        raise MASTError(self.__class__.__name__, "This function is obsolete.")
        return self.ingredients.get(ingredient_name)

    #def add_parent(self, ingredient_name, parent_name):
    #    """Used to build the dependency dict using the parent child relationship
    #    """
    #    self.dependency_dict.setdefault(ingredient_name, list()).append(parent_name)

    def ingredient_iterator(self):
        """Iterates through the ingredients dict and returns the ingredients one by one
        """
        raise MASTError(self.__class__.__name__, "This function is obsolete.")
        for ingredient_name, ingredient_obj in self.ingredients.iteritems():
            yield ingredient_name, ingredient_obj

    def __iter__(self):
        """Iterates through the ingredients dict and returns the ingredients one by one
        """
        raise MASTError(self.__class__.__name__, "This function is obsolete.")
        for ingredient_name, ingredient_obj in self.ingredients.iteritems():
            yield ingredient_obj
    def __repr__(self):
        """Print information."""
        rlines=""
        rlines=rlines + "Recipe name: %s\n" % self.name
        rlines=rlines + "Ingredients: \n"
        ikeys = self.ingredients.keys()
        ikeys.sort()
        for ikey in ikeys:
            rlines=rlines + "    %s\n" % ikey
            rlines=rlines + "        Parents: \n" 
            for myparent in self.parents_to_check[ikey]:
                rlines=rlines + "            %s\n" % myparent
            rlines=rlines + "        Write:    %s\n" % self.write_methods[ikey]
            rlines=rlines + "        Ready:    %s\n" % self.ready_methods[ikey]
            rlines=rlines + "        Run:      %s\n" % self.run_methods[ikey]
            rlines=rlines + "        Complete: %s\n" % self.complete_methods[ikey]
            rlines=rlines + "        Children: \n"
            for htukey in self.update_methods[ikey]:
                rlines=rlines + "            %s:%s\n" % (htukey, self.update_methods[ikey][htukey])
        self.print_status()
        return rlines
    def get_statuses_from_file(self):
        """Get status of each ingredient from a status.txt
            file in the recipe directory.
        """
        statpath = os.path.join(self.working_directory,'status.txt')
        if not os.path.isfile(statpath):
            raise MASTError(self.__class__.__name__, "Could not get status file in %s" % statpath)
        statfile = MASTFile(statpath)
        for statline in statfile.data:
            if statline[0] == "#":
                pass
            else:
                statsplit = statline.strip().split(':')
                oneingred = statsplit[0].strip()
                onestatus = statsplit[1].strip()
                if oneingred in self.ingredients.keys():
                    self.ingredients[oneingred] = onestatus
                else:
                    raise MASTError(self.__class__.__name__, "Ingredient %s is not in the original recipe's ingredients list." % oneingred)



import random

def tournament1(pop, nkeep, Optimizer):
    """Selection function that orders and chooses individuals based on tournament schemes
    Function will attempt to prevent repeats
    """
    tournsize = Optimizer.tournsize
    fits = [ind.fitness for ind in pop]
    minindex = min(xrange(len(fits)), key=fits.__getitem__)
    newpop = []
    indices = []
    indexo = [ind.index for ind in pop]
    for i in range(len(pop)):
        pop[i].index = i
    counter=0
    while len(newpop)< nkeep:
        subgroup = random.sample(pop,tournsize)
        mine = min([ind.fitness for ind in subgroup])
        select = [ind for ind in subgroup if ind.fitness==float(mine)][0]
        if select.index not in indices:
            newpop.append(select)
            indices.append(select.index)
        elif counter > 100000:
            try:
                nonpop = [ind for ind in pop if ind.index not in indices]
                select = random.choice(nonpop)
                newpop.append(select)
                indices.append(select.index)
            except:
                select = random.choice(pop)
                newpop.apend(select)
                indices.append(select.index)
        counter+=1
    for i in range(len(newpop)):
        newpop[i].index = indexo[indices[i]]
    if minindex not in indices:
        rm = random.choice(indices)
        newpop = [pop[inx] for inx in indices if inx != rm]
        newpop.append(pop[minindex])
    return newpop


import random

def swap_vacancy(indiv, Optimizer):
    """Move function to perform mutation to swap the vacancy position with another atom position
    Inputs:
        indiv = Individual class object to be altered
        Optimizer = Optimizer class object with needed parameters
    Outputs:
        indiv = Altered Individual class object
    """
    if 'MU' in Optimizer.debug:
        debug = True
    else:
        debug = False
    Optimizer.output.write('Vacancy Swap Mutation performed on individual\n')
    Optimizer.output.write('Index = '+repr(indiv.index)+'\n')
    vacancies = indiv.vacancies.copy()
    if len(vacancies) == 0:
        Optimizer.output.write('Vacancy Swap Failed. Individual has no identified vacancies to exchange\n')
        natomsswap = 0
    elif len(vacancies) == 1:
        natomsswap = 1
    else:
        natomsswap = random.randint(1,len(vacancies))
    str=[]
    for one in range(natomsswap):
        vacant = random.choice(vacancies)
        if Optimizer.alloy:
            solid=indiv[0].copy()
            solid.extend(indiv.bulki)
            select = random.choice([one.index for one in solid if one.symbol==vacant.symbol])
        else:
            select = random.choice([one.index for one in indiv[0] if one.symbol==vacant.symbol])
        if select < len(indiv[0]):
            indiv[0][select].position, vacancies[vacant.index].position = vacant.position, indiv[0][select].position
            str.append('C')
        else:
            indiv.bulki[select-len(indiv[0])].position, vacancies[vacant.index].position = vacant.position, indiv.bulki[select-len(indiv[0])].position
            str.append('B')
    indiv.vacancies = vacancies
    Optimizer.output.write('Exchanged the position of '+repr(natomsswap)+' vacancies\n')
    cex = len([one for one in str if one=='C'])
    bex = len([one for one in str if one=='B'])
    Optimizer.output.write('Cluster exchanges: '+repr(cex)+'\n')
    Optimizer.output.write('Bulk exchanges: '+repr(bex)+'\n')
    muttype='VS'+repr(natomsswap)+'c'+repr(cex)+'b'+repr(bex)
    if indiv.energy==0:
        indiv.history_index=indiv.history_index+'m'+muttype
    else:
        indiv.history_index=repr(indiv.index)+'m'+muttype
    return indiv


from MAST.structopt.tools import get_best
from MAST.structopt.switches import selection_switch
from MAST.structopt.switches import lambdacommamu
from MAST.structopt.tools import remove_duplicates
import logging
import random
import pdb

def predator_switch(pop,Optimizer):
    """Function for removing individuals from the population"""
    #logger = initialize_logger(Optimizer.loggername)
    logger = logging.getLogger(Optimizer.loggername)
    scheme = Optimizer.predator
    logger.info('Applying predator to population with initial size = {0}'.format(len(pop)))
    STR = 'PREDATOR\n'
    try:
       exec "from MAST.structopt.predator.{0} import {0}".format(scheme)
       pop, STR = eval('{0}(pop, Optimizer)'.format(scheme))
       passflag = True
    except NameError, e:
        logger.warning('Specified predator not one of the available options. Please check documentation and spelling! Predator : {0}. {1}'.format(scheme,e), exc_info=True)
        passflag = False
        STR+='Specified predator not one of the available options. Please check documentation and spelling! Predator : '+repr(scheme)
        STR+=repr(e)+'\n'
    except Exception, e:
        logger.error('ERROR: Issue in Predator Scheme. Predator = {0}. {1}'.format(scheme,e), exc_info=True)
        print 'ERROR: Issue in Predator Scheme. Predator = '+repr(scheme)
        print e
        passflag = False
        STR+=''
    if not passflag:
        logger.warning('Issue in predator. Attempting basic Fitpred')
        fitlist = [one.fitness for one in pop]
        nfitlist, nindices = remove_duplicates(fitlist, Optimizer.demin)
        STR += 'Issue in predator. Attempting basic Fitpred\n'
        newpop = []
        if len(nfitlist) != len(fitlist):
            STR+='Predator: Removed total of '+repr(len(fitlist)-len(nfitlist))+' from population\n'
        otherlist = []
        for i in range(len(pop)):
            if i not in nindices:
                STR+='Predator: Removed '+repr(pop[i].history_index)+'\n'
                otherlist.append(pop[i])
            else:
                newpop.append(pop[i])
        while len(newpop) < Optimizer.nindiv:
            STR+='Predator: Adding duplicates back\n'
            choice = random.choice(otherlist)
            if choice.index not in nindices:
                newpop.append(choice)
                nindices.append(choice.index)
        nindices.sort()
        if Optimizer.natural_selection_scheme=='fussf':
            for ind in newpop:
                if ind.fingerprint == 0:
                    ind.fingerprint = get_fingerprint(Optimizer,ind,Optimizer.fpbin,Optimizer.fpcutoff)
        if 'lambda,mu' in Optimizer.algorithm_type:
            try:
                mark = [ index for index,n in enumerate(nindices) if n > Optimizer.nindiv-1][0]
            except:
                mark = Optimizer.nindiv
            Optimizer.mark = mark
            pop,str = lambdacommamu.lambdacommamu(newpop, Optimizer)
            STR+=str
        else:
            pop = selection_switch(newpop, Optimizer.nindiv, Optimizer.natural_selection_scheme, Optimizer)
        pop = get_best(pop,len(pop))
    Optimizer.output.write(STR)
    return pop




import random
import numpy
import copy
from ase import Atom, Atoms
from ase.calculators.neighborlist import NeighborList

def swap_int_local(indiv, Optimizer):
    """Move function to perform n atom structure swap based on swaplist for cluster in local region
    Inputs:
        indiv = Individual class object to be altered
        Optimizer = Optimizer class object with needed parameters
    Outputs:
        indiv = Altered Individual class object
    """
    if 'MU' in Optimizer.debug:
        debug = True
    else:
        debug = False
    Optimizer.output.write('Local Cluster Swap Mutation performed on individual\n')
    Optimizer.output.write('Index = '+repr(indiv.index)+'\n')
    # Identify the local atoms available for swapping
    atmsst = indiv[0].copy()
    nat = len(atmsst)
    totalsol = atmsst.copy()
    if Optimizer.structure=='Defect':
        totalsol.extend(indiv.bulki.copy())
        ctoff = [Optimizer.sf for one in totalsol]
    else:
        ctoff = [1.5 for one in totalsol]
    nl = NeighborList(ctoff, bothways=True, self_interaction=False)
    nl.update(totalsol)
    cluster = Atoms()
    indices = [] #store indices of the atoms for later
    for i in range(nat):
        indices1, offsets = nl.get_neighbors(i)
        for index, d in zip(indices1,offsets):
            index = int(index)
            #Make sure to prevent repeats of the same atom
            if index not in indices:
                pos = totalsol[index].position + numpy.dot(d,totalsol.get_cell())
                cluster.append(Atom(symbol=totalsol[index].symbol,position=pos))
                indices.append(index)
    # Identify Number of atoms to swap
    swapmax=sum([c for sym,c in indiv.swaplist])
    if swapmax >1:
        natomsswap=random.randint(1,swapmax)
    elif swapmax==1:
        natomsswap=1
    else:
        Optimizer.output.write('Maximum number of swaps is '+repr(swapmax)+'. Unable to perform mutation')
        natomsswap=0
    if len(indiv.swaplist)==1:
        Optimizer.output.write('WARNING: Swap Mutation attempted on single atom structure system\n')
        natomsswap=0
    Optimizer.output.write('Number of swaps = '+repr(natomsswap)+'\n')
    # Identify symbols that can be swapped
    syms=[sym for sym,c in indiv.swaplist]
    slist = copy.deepcopy(indiv.swaplist)
    if debug: print 'Starting swaplist = '+repr(indiv.swaplist)
    #Sanity check
    sanch = [[sym,0] for sym in syms]
    for i in range(len(sanch)):
        nc = len([atm for atm in cluster if atm.symbol==sanch[i][0]])
        nc += [c for sym,c in slist if sym==sanch[i][0]][0]
        sanch[i][1]=nc
    # Swap Atoms
    for i in range(natomsswap):
        while True:
            at1 = random.choice(cluster)
            osym = at1.symbol
            nsyml=[sym for sym,c in slist if sym != osym and c > 0]
            if len(nsyml) > 0:
                break
        nsym = random.choice(nsyml)
        cluster[at1.index].symbol = nsym
        Optimizer.output.write('Swapped '+nsym+' atom with '+osym+'\n')
        for i in range(len(slist)):
            if slist[i][0]==nsym:
                slist[i][1]-=1
            elif slist[i][0]==osym:
                slist[i][1]+=1
    # Apply the new swap to the actual atoms in the structure
    for i in range(len(indices)):
        totalsol[indices[i]].symbol = cluster[i].symbol
    # Separate out bulk from defects
    indiv[0] = totalsol[0:nat]
    if Optimizer.structure=='Defect':
        indiv.bulki = totalsol[nat::]
    # Update new swaplist
    indiv.swaplist = slist
    #Sanity check
    sanchn = [[sym,0] for sym in syms]
    for i in range(len(sanchn)):
        nc = len([atm for atm in cluster if atm.symbol==sanchn[i][0]])
        nc += [c for sym,c in slist if sym==sanchn[i][0]][0]
        sanchn[i][1]=nc
    if debug:
        for i in range(len(sanch)):
            print 'INTSWAP: Starting Atom structure '+repr(sanch[i][0])+' = '+repr(sanch[i][1])
            print 'INTSWAP: After Mutation Atom structure '+repr(sanchn[i][0])+' = '+repr(sanchn[i][1])
    Optimizer.output.write(repr(indiv[0])+'\n')
    muttype='SCL'+repr(natomsswap)
    if indiv.energy==0:
        indiv.history_index=indiv.history_index+'m'+muttype
    else:
        indiv.history_index=repr(indiv.index)+'m'+muttype
    return indiv


import random
import numpy
from ase import Atom, Atoms
from ase.calculators.neighborlist import NeighborList
from MAST.structopt.inp_out import write_xyz

def newclus(ind1, ind2, Optimizer):
    """Select a box in the cluster configuration"""
    if 'CX' in Optimizer.debug:
        debug = True
    else:
        debug = False
    Optimizer.output.write('Box Cluster Cx between individual '+repr(ind1.index)+' and individual '+repr(ind2.index)+'\n')
    
    #Perserve starting conditions of individual
    solid1 = ind1[0].copy()
    solid2 = ind2[0].copy()
    cello1 = ind1[0].get_cell()
    cello2 = ind2[0].get_cell()
    cell1 = numpy.maximum.reduce(solid1.get_positions())
    cell1m = numpy.minimum.reduce(solid1.get_positions())
    cell2 = numpy.maximum.reduce(solid2.get_positions())
    cell2m = numpy.minimum.reduce(solid2.get_positions())
    cell = numpy.minimum(cell1,cell2)
    pbc1 = solid1.get_pbc()
    pbc2 = solid2.get_pbc()
    #Get starting concentrations and number of atoms
    nat1 = len(solid1)
    nat2 = len(solid2)

    # Pick a origin point for box in the cell
    pt1 = random.choice(solid1)
    pt1f = [(pt1.position[i]-cell1m[i])/cell1[i] for i in range(3)]
    pt2 = [pt1f[i]*cell2[i]+cell2m[i] for i in range(3)]
    solid2.append(Atom(position=pt2))
    pt2=solid2[len(solid2)-1]
    #Find max neighborsize of circle cut
    r = random.uniform(0,min(nat1,nat2)/5.0)
    if debug:
        print 'DEBUG CX: Point one =', pt1.position
        print 'DEBUG CX: Point two =', pt2.position
    #Find atoms within sphere of neighborsize r for both individuals
    #Make sure that crossover is only selection of atoms not all
    while True:
        ctoff = [r for on in solid1]
        nl = NeighborList(ctoff, bothways=True, self_interaction=False)
        nl.update(solid1)
        indices1, offsets = nl.get_neighbors(pt1.index)
        if len(indices1)==0:
            r = r*1.2
        elif len(indices1) < nat1*.75:
            break
        else:
            r = r*0.8
    if debug:
        print 'Neighborsize of box = '+repr(r)+'\nPosition in solid1 = '+repr(pt1.position)+'\nPosition in solid2 = '+repr(pt2.position)
    group1 = Atoms(cell=solid1.get_cell(),pbc=solid1.get_pbc())
    group1.append(pt1)
    indices1a=[pt1.index]
    for index, d in zip(indices1,offsets):
        if index not in indices1a:
            index = int(index)
            pos = solid1[index].position + numpy.dot(d,solid1.get_cell())
            group1.append(Atom(symbol=solid1[index].symbol,position=pos))
            indices1a.append(index)
    indices1=indices1a
    ctoff = [r for on in solid2]
    nl = NeighborList(ctoff, bothways=True, self_interaction=False)
    nl.update(solid2)
    indices2, offsets = nl.get_neighbors(pt2.index)
    group2 = Atoms(cell=solid2.get_cell(),pbc=solid2.get_pbc())
    indices2a = []
    for index, d in zip(indices2,offsets):
        if index not in indices2a:
            index = int(index)
            pos = solid2[index].position + numpy.dot(d,solid2.get_cell())
            group2.append(Atom(symbol=solid2[index].symbol,position=pos))
            indices2a.append(index)
    indices2=indices2a
    if len(indices2)==0:
        for one in group1:
            while True:
                sel = random.choice(solid2)
                if sel.symbol==one.symbol:
                    if sel.index not in indices2:
                        group2.append(sel)
                        indices2.append(sel.index)
                        break

    if Optimizer.forcing=='Concentration':
        symlist = list(set(group1.get_chemical_symbols()))
        seplist = [[atm for atm in group2 if atm.symbol == sym] for sym in symlist]
        group2n=Atoms(cell=group2.get_cell(),pbc=group2.get_pbc())
        indices2n = []
        dellist = []
        for one in group1:
            sym1=one.symbol
            listpos=[i for i,s in enumerate(symlist) if s==sym1][0]
            if len(seplist[listpos]) > 0:
                pos = random.choice(range(len(seplist[listpos])))
                group2n.append(seplist[listpos][pos])
                indices2n.append(indices2[seplist[listpos][pos].index])
                del seplist[listpos][pos]
            else:
                dellist.append(one.index)
        if len(dellist) != 0:
            dellist.sort(reverse=True)
            for one in dellist:
                del group1[one]
                del indices1[one]
        indices2n.append(pt2.index)
        indices2=indices2n
        group2=group2n.copy()
    else:
        dellist = []
        while len(group2) < len(group1)-len(dellist):
            #Too many atoms in group 1
            dellist.append(random.choice(group1).index)
        if len(dellist) != 0:
            dellist.sort(reverse=True)
            for one in dellist:
                del group1[one]
                del indices1[one]
        dellist = []
        while len(group1) < len(group2)-len(dellist):
            #Too many atoms in group 2
            dellist.append(random.choice(group2).index)
        if len(dellist) != 0:
            dellist.sort(reverse=True)
            for one in dellist:
                del group2[one]
                del indices2[one]
            
    other2 = Atoms(cell=solid2.get_cell(),pbc=solid2.get_pbc())
    for one in solid2:
        if one.index not in indices2:
            other2.append(one)
    other1 = Atoms(cell=solid1.get_cell(),pbc=solid1.get_pbc())
    for one in solid1:
        if one.index not in indices1:
            other1.append(one)

    #Exchange atoms in sphere and build new solids
    nsolid1 = other1.copy()
    nsolid1.extend(group2.copy())
    nsolid2 = other2.copy()
    nsolid2.extend(group1.copy())

    #DEBUG: Write crossover to file
    if debug: 
        write_xyz(Optimizer.debugfile, nsolid1,'CX(randalloybx):nsolid1')
        write_xyz(Optimizer.debugfile, nsolid2,'CX(randalloybx):nsolid2')


    #DEBUG: Check structure of atoms exchanged
    for sym,c,m,u in Optimizer.atomlist:
        if Optimizer.structure=='Defect':
            nc=len([atm for atm in nsolid1 if atm.symbol==sym])
            nc+=len([atm for atm in ind1.bulki if atm.symbol==sym])
            oc=len([atm for atm in solid1 if atm.symbol==sym])
            oc+=len([atm for atm in ind1.bulki if atm.symbol==sym])
        else:
            nc=len([atm for atm in nsolid1 if atm.symbol==sym])
            oc=len([atm for atm in solid1 if atm.symbol==sym])
        Optimizer.output.write('CX(clustbx):New solid1 contains '+repr(nc)+' '+repr(sym)+' atoms\n')
        if debug: print 'DEBUG CX: New solid1 contains '+repr(nc)+' '+repr(sym)+' atoms'
        if oc != nc:
            #pdb.set_trace()
            print 'CX: Issue in maintaining atom concentration\n Dropping new individual'
            Optimizer.output.write('CX: Issue in maintaining atom concentration\n Dropping new individual 1\n')
            nsolid1 = solid1
    for sym,c,m,u in Optimizer.atomlist:
        if Optimizer.structure=='Defect':
            nc=len([atm for atm in nsolid2 if atm.symbol==sym])
            nc+=len([atm for atm in ind2.bulki if atm.symbol==sym])
            oc=len([atm for atm in solid2 if atm.symbol==sym])
            oc+=len([atm for atm in ind2.bulki if atm.symbol==sym])
        else:
            nc=len([atm for atm in nsolid2 if atm.symbol==sym])
            oc=len([atm for atm in solid2 if atm.symbol==sym])
        Optimizer.output.write('CX(clustbx):New solid2 contains '+repr(nc)+' '+repr(sym)+' atoms\n')
        if debug: print 'DEBUG CX: New solid2 contains '+repr(nc)+' '+repr(sym)+' atoms'
        if oc != nc:
            #pdb.set_trace()
            print 'CX: Issue in maintaining atom concentration\n Dropping new individual'
            Optimizer.output.write('CX: Issue in maintaining atom concentration\n Dropping new individual 2\n')
            solid2.pop()
            nsolid2 = solid2
    if Optimizer.forcing !='Concentration':
        for i in range(len(Optimizer.atomlist)):
            atms1=[inds for inds in nsolid1 if inds.symbol==Optimizer.atomlist[i][0]]
            atms2=[inds for inds in nsolid2 if inds.symbol==Optimizer.atomlist[i][0]]
            if len(atms1)==0:
                if len(atms2)==0:
                    nsolid1[random.randint(0,len(indi1)-1)].symbol==Optimizer.atomlist[i][0]
                    nsolid2[random.randint(0,len(indi2)-1)].symbol==Optimizer.atomlist[i][0]
                else:
                    nsolid1.append(atms2[random.randint(0,len(atms2)-1)])
                    nsolid1.pop(random.randint(0,len(nsolid1)-2))
            else:
                if len(atms2)==0:
                    nsolid2.append(atms1[random.randint(0,len(atms1)-1)])
                    nsolid2.pop(random.randint(0,len(nsolid2)-2))	

    nsolid1.set_cell(cello1)
    nsolid2.set_cell(cello2)
    nsolid1.set_pbc(pbc1)
    nsolid2.set_pbc(pbc2)
    
    ind1[0]=nsolid1.copy()
    ind2[0]=nsolid2.copy()

    return ind1, ind2
